#coding:utf-8

from adV2 import MADAPI as MAD
from adV2.api.APIException import APIException as APIException
from adV2.api.ErrorInfo import ErrInfo
from adV2.api.NetnsInfo import NetnsInfo #, NetnsInfoList
from adV2.api.MacvlanInfo import MacvlanInfo #, MacvlanInfoList
from adV2.api.VlanInfo import VlanInfo, VlanInfoList
from adV2.api.VxlanInfo import VxlanInfo, VxlanList
from adV2.api.VxnetInfo import VxnetInfo, VxnetList
from adV2.api.ServInfo import ServInfo, ServInfoList, PortInfo
from adV2.api.IPGroup import IPGroup, IPGroupList, IPInfo
from adV2.api.PersistInfo import PersistInfo, PersistList
from adV2.api.MonitorInfo import MonitorInfo, MonitorList
from adV2.api.NodeInfo import NodeInfo, NodeList
from adV2.api.PoolInfo import PoolInfo, PoolList
from adV2.api.VSInfo import VSInfo, VSList, VSStatus
from adV2.api.WanInfo import WanInfo, WanList
from adV2.api.LanInfo import LanInfo, LanList
from adV2.api.AclInfo import AclInfo, AclList
from adV2.api.SnatSet import SnatSet, SnatSetList
from adV2.api.AddrElement import AddrElement
from adV2.api.StaticRouteInfo import StaticRouteInfo
from adV2.api.SchedRule import *
from oslo_log import log as logging
from oslo_log import helpers as log_helpers
import traceback
import urllib

#MADAPI=MAD.MADAPI
ADAPI=MAD.ADAPI

#会话保持默认超时时间 单位：秒
DEFAULT_TIMEOUT = 86400
#节点默认最大权值
DEFAULT_WEIGHT_MAX = 100
#连接限制最大值
DEFAULT_CONN_LIMIT = 2147483647

LOG = logging.getLogger(__name__)

class ADHOST(object):
    '''
    AD主机操作类
    '''

    def __init__(self, username, password, host, port):
        self.adapi = ADAPI(username, password, host, port, 60)

    def __configExist(self, code):
        if code == 105:
            return False
        else:
            return True

    @log_helpers.log_method_call
    def checkService(self, pbdata):
        try:
            return self.adapi.checkService(pbdata)
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise e

    @log_helpers.log_method_call
    def createNetns(self, name, tenant, netns_uuid="0", conns=0, cps=0, flow=0):
        """创建netns"""
        netnsInfo = NetnsInfo()
        netnsInfo.name = name
        netnsInfo.tenant = tenant
        netnsInfo.netns_uuid = netns_uuid
        netnsInfo.conns_limit = conns
        netnsInfo.new_conns_limit = cps
        netnsInfo.flows_limit = flow
        try:
            ret = self.adapi.createNetns(netnsInfo)
            if ret.isFailed():
                # 创建失败时，查看一下netns是否已经存在，存在则不抛异常
                netns = self.adapi.getNetnsInfo(name)
                if netns and netns.isFailed() == False:
                    return
                raise Exception('createNetns failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteNetns(self, name):
        '''
        删除netns配置
        '''
        try:
            ret = self.adapi.deleteNetns(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteNetns failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateNetns(self, oldname, name, tenant):
        '''
        更新netns配置信息
        '''
        netnsInfo = NetnsInfo()
        netnsInfo.name = name
        netnsInfo.tenant = tenant
        try:
            ret = self.adapi.updateNetns(oldname, netnsInfo)
            if ret.isFailed():
                raise Exception('updateNetns failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def getNetnsInfo(self, name):
        '''
        获取netns配置信息
        '''
        netns = {}
        try:
            netnsInfo = self.adapi.getNetnsInfo(name)
            if netnsInfo.isFailed():
                raise Exception('getNetnsinfo failed[%d, %s]' % (netnsInfo.getErrCode(),
                                                              netnsInfo.getErrStr()))
            netns['name'] = netnsInfo.name
            netns['netnsId'] = netnsInfo.netnsId
            netns['tenant'] = netnsInfo.tenant
            netns['appgId'] = netnsInfo.appgId
        except Exception as e:
            LOG.error("getNetnsInfo Exception: %s" % traceback.format_exc())
            #raise Exception(e)
            return None
        return netns

    @log_helpers.log_method_call
    def getNetnsId(self, name):
        netns = self.getNetnsInfo(name)
        if netns:
            return netns['netnsId']
        return None

    @log_helpers.log_method_call
    def createMacvlan(self, name, mac, interface, netns_id, netns_uuid = "0"):
        """创建macvlan"""
        macvlanInfo = MacvlanInfo()
        macvlanInfo.name = name
        macvlanInfo.mac = mac
        macvlanInfo.ifname = interface
        macvlanInfo.netnsId = netns_id
        macvlanInfo.netns_uuid = netns_uuid
        try:
            ret = self.adapi.createMacvlan(macvlanInfo)
            if ret.isFailed():
                # 创建失败时，查看一下macvlan是否已经存在，存在则不抛异常
                macvlan = self.adapi.getMacvlanInfo(name)
                if macvlan.isFailed() == False:
                    return
                raise Exception('createMacvlan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteMacvlan(self, name):
        '''
        删除macvlan配置
        '''
        try:
            ret = self.adapi.deleteMacvlan(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteMacvlan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateMacvlan(self, oldname, name, mac, interface, netns_id):
        '''
        更新macvlan配置信息
        '''
        macvlanInfo = MacvlanInfo()
        macvlanInfo.name = name
        macvlanInfo.mac = mac
        macvlanInfo.ifname = interface
        macvlanInfo.netnsId = netns_id
        try:
            ret = self.adapi.updateMacvlan(oldname, macvlanInfo)
            if ret.isFailed():
                raise Exception('updateMacvlan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def getMacvlanInfo(self, name):
        '''
        获取macvlan配置信息
        '''
        macvlan = {}
        try:
            macvlanInfo = self.adapi.getMacvlanInfo(name)
            macvlan['name'] = macvlanInfo.name
            macvlan['mac'] = macvlanInfo.mac
            macvlan['ifname'] = macvlanInfo.ifname
            macvlan['netnsId'] = macvlanInfo.netnsId
            macvlan['device'] = macvlanInfo.device
            if macvlanInfo.isFailed():
                raise Exception('getMacvlaninfo failed[%d, %s]' % (macvlanInfo.getErrCode(),
                                                              macvlanInfo.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)
        return macvlan

    @log_helpers.log_method_call
    def getMacvlanDevice(self, name):
        macvlan = self.getMacvlanInfo(name)
        return macvlan['device']

    @log_helpers.log_method_call
    def createVlan(self, name, id, interface='eth1'):
        """创建vlan"""
        vlanInfo = VlanInfo()
        vlanInfo.name = name
        vlanInfo.ifname = interface
        vlanInfo.vlanId = id
        try:
            ret = self.adapi.createVlan(vlanInfo)
            if ret.isFailed():
                # 创建失败时，查看一下vlan是否已经存在，存在则不抛异常
                vlan = self.adapi.getVlanInfo(name)
                if vlan.isFailed() == False:
                    return
                raise Exception('createVlan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __vlanNotUse(self, ret):
        '''
        是否vlan未被使用
        用于删除vlan接口，使用中的vlan删除失败常
        '''
        if ret.getErrCode() == 212 :
            if "占用" in ret.getErrStr():
                return False

        return True

    @log_helpers.log_method_call
    def deleteVlan(self, name):
        '''
        删除vlan配置
        '''
        try:
            ret = self.adapi.deleteVlan(name)
            if ret.isFailed() and self.__vlanNotUse(ret):
                raise Exception('deleteVlan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateVlan(self, oldname, name, id, interface='eth1'):
        '''
        更新vlan配置信息
        '''
        vlanInfo = VlanInfo()
        vlanInfo.name = name
        vlanInfo.ifname = interface
        vlanInfo.vlanId = id
        try:
            ret = self.adapi.updateVlan(oldname, vlanInfo)
            if ret.isFailed():
                raise Exception('updateVlan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def getVlanInfo(self, name):
        '''
        获取vlan配置信息
        '''
        vlan = {}
        try:
            vlanInfo = self.adapi.getVlanInfo(name)
            vlan['name'] = vlanInfo.name
            vlan['vlanId'] = vlanInfo.vlanId
            vlan['ifname'] = vlanInfo.ifname
            vlan['device'] = vlanInfo.device
            if vlanInfo.isFailed():
                raise Exception('getVlaninfo failed[%d, %s]' % (vlanInfo.getErrCode(),
                                                              vlanInfo.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)
        return vlan

    @log_helpers.log_method_call
    def getVlanDevice(self, name):
        vlan = self.getVlanInfo(name)
        return vlan['device']

    @log_helpers.log_method_call
    def __buildVxlanInfo(self, name, vxlan_id, vxlan_domain):
        vxlan = VxlanInfo()
        vxlan.name = name
        vxlan.vxlan_id = vxlan_id
        vxlan.vxlan_domain = vxlan_domain
        # vxlan.device = 'vx%d.%d'
        return vxlan

    @log_helpers.log_method_call
    def createVxlan(self, name, vxlan_id, vxlan_domain=0):
        '''
        创建vxlan配置
        '''
        try:
            vxlan = self.__buildVxlanInfo(name, vxlan_id, vxlan_domain)
            ret = self.adapi.createVxlan(vxlan)
            if ret.isFailed():
                # 创建失败时，查看一下vxlan是否已经存在，存在则不抛异常
                vxlanInfo = self.adapi.getVxlanInfo(name)
                if vxlanInfo.isFailed() == False:
                    return
                raise Exception('createVxlan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteVxlan(self, name):
        '''
        删除vxlan配置
        '''
        try:
            ret = self.adapi.deleteVxlan(name)
            if ret.isFailed():
                raise Exception('deleteVxlan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateVxlan(self, oldname, name, vxlan_id, vxlan_domain=0):
        '''
        更新vxlan配置
        '''
        try:
            vxlan = self.__buildVxlanInfo(name, vxlan_id, vxlan_domain)
            ret = self.adapi.updateVxlan(oldname, vxlan)
            if ret.isFailed():
                raise Exception('updateVxlan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def getVxlaninfo(self, name):
        '''
        获取vxlan配置信息
        '''
        vxlan = {}
        try:
            vxlanInfo = self.adapi.getVxlanInfo(name)
            vxlan['name'] = vxlanInfo.name
            vxlan['vxlan_id'] = vxlanInfo.vxlan_id
            vxlan['device'] = vxlanInfo.device
            vxlan['vxlan_domain'] = vxlanInfo.vxlan_domain
            if vxlanInfo.isFailed():
                raise Exception('getVxlaninfo failed[%d, %s]' % (vxlanInfo.getErrCode(),
                                                        vxlanInfo.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)
        return vxlan

    @log_helpers.log_method_call
    def getVxlanDevice(self, name):
        vxlan = self.getVxlaninfo(name)
        return vxlan['device']

    @log_helpers.log_method_call
    def __buildVxnetInfo(self, name, port, vteps, vxnet_domain):
        vxnet = VxnetInfo()
        vxnet.name = name
        vxnet.vxnet_domain = vxnet_domain
        vxnet.port = port
        vxnet.vteps = vteps
        return vxnet

    @log_helpers.log_method_call
    def createVxnet(self, name, port, vteps, vxnet_domain=0):
        '''
        创建vxnet配置
        '''
        try:
            vxnet = self.__buildVxnetInfo(name, port, vteps, vxnet_domain)
            ret = self.adapi.createVxnet(vxnet)
            if ret.isFailed():
                # 创建失败时，查看一下vxnet是否已经存在，存在则不抛异常
                vxnetInfo = self.adapi.getVxnetInfo(name)
                if vxnetInfo.isFailed() == False:
                    return
                raise Exception('createVxnet failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteVxnet(self, name):
        '''
        删除vxnet配置
        '''
        try:
            ret = self.adapi.deleteVxnet(name)
            if ret.isFailed():
                raise Exception('deleteVxnet failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateVxnet(self, oldname, name, port, vteps, vxnet_domain=0):
        '''
        更新vxnet配置
        '''
        try:
            vxnet = self.__buildVxnetInfo(name, port, vteps, vxnet_domain)
            ret = self.adapi.updateVxnet(oldname, vxnet)
            if ret.isFailed():
                raise Exception('updateVxnet failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def getVxnetinfo(self, name):
        try:
            vxlaninfo = self.adapi.getVxnetInfo(name)
        except Exception:
            vxlaninfo = None
        if vxlaninfo and vxlaninfo.isFailed():
            vxlaninfo = None
        return vxlaninfo

    def __buildServInfo(self, name, type="SRV_HTTP", port=[], netns_uuid = None):
        srv = ServInfo()
        srv.name = name
        # openstack TERMINATED_HTTPS/HTTPS/TCP 负载均认为TCP 负载
        # 原因openstack无法配置https卸载
        srv.type = type
        srv.netns_uuid = netns_uuid
        for new_prot in port:
            portInfo = PortInfo()
            portInfo.fromPort = int(new_prot)
            srv.portInfo.append(portInfo)
        return srv


    @log_helpers.log_method_call
    def createServ(self, name, Stype="SRV_HTTP", port=[], netns_uuid = "0"):
        '''
        创建服务配置
        @name: 名字
        @type: 负载均衡的类型
        @port: 端口号数组
        @return:
        '''
        try:
            srv = self.__buildServInfo(name, Stype, port, netns_uuid)
            ret = self.adapi.createServInfo(srv)
            if ret.isFailed():
                info = self.adapi.getServInfo(name)
                if info.isFailed() == False:
                    return
                raise Exception('createServ failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateServ(self, name, type="SRV_HTTP", port=[]):
        '''
        添加端口号
        @param name:
        @param type:
        @param port 端口列表, 注意：一次只能添加一个
        @return:
        '''
        try:
            srv = self.__buildServInfo(name, type, port)
            srv.type = None       # 更新不需要类型
            ret = self.adapi.addServPort(srv)
            if ret.isFailed():
                raise Exception('updateServ failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteServ(self, name):
        '''
        删除服务配置
        '''
        try:
            ret = self.adapi.delServInfo(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteServ failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __buildGroupInfo(self, name, ipg=[], netns_uuid=None):
        ipgroup = IPGroup()
        ipgroup.name = name
        ipgroup.netns_uuid = netns_uuid
        ipInfo = IPInfo()
        ipInfo.ipAddr = ipg
        ipgroup.ipgInfo.append(ipInfo)
        return ipgroup

    @log_helpers.log_method_call
    def createIPGroup(self, name, ip=[], args={}, netns_uuid = '0'):
        '''
        构建AD ip组+ ip 配置块
        @param name:
        @param ip:  ip组数组
        @return:
        '''
        if 'wan' not in args:
            LOG.error("no wan name")
            raise Exception()
        try:
            # 先创建WAN口配置
            self.createWan(args['wan'], args['ifname'], ip, args, netns_uuid)

            # 再创建IP组
            ipgroup = self.__buildGroupInfo(name, ip, netns_uuid)
            ret = self.adapi.createIPGroup(ipgroup)
            if ret.isFailed():
                info = self.adapi.getIPGroup(name)
                if info.isFailed() == False:
                    return
                raise Exception('createIPGroup failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateIPGroup(self, name, ip=[]):
        '''
        添加ip组
        @param name:
        @param ip:
        @return:
        '''
        LOG.debug("CREATE")
        try:
            ipgroup = self.__buildGroupInfo(name, ip)
            ret = self.adapi.addIPInfo(ipgroup)
            if ret.isFailed():
                raise Exception('updateIPGroup failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteIPGroup(self, name):
        '''
        删除IP组配置
        '''
        try:
            # 删除IP组
            ret = self.adapi.delIPGroup(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteIPGroup failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def IPGroupAddIP(self, ipgrpname="", ip=[]):
        '''
        往IP组内添加IP
        '''
        try:
            # 先添加Wan口IP
            self.addWanIP(ipgrpname, ip)
            # 添加IP组IP
            ipgroup = self.__buildGroupInfo(ipgrpname, ip)
            ret = self.adapi.addIPInfo(ipgroup)
            if ret.isFailed():
                raise Exception('IPGroupAddIP failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def IPGroupDelIP(self, ipgrpname="", ip=[]):
        '''
        从IP组中删除IP
        '''
        try:
            # 先删除WAN口ip
            self.delWanIP(ipgrpname, ip)
            #在删除IP组IP
            ipgroup = self.__buildGroupInfo(ipgrpname, ip)
            ret = self.adapi.delIPInfo(ipgroup)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('IPGroupDelIP failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def __buildPersist(self, name, kargs={}, netns_uuid = None):
        '''
        构建AD 会话保持配置块
        @param name:
        @param kargs:
        @return:
        '''
        persist = PersistInfo()
        persist.name = name
        persist.priorToConnect = kargs.get('priorToConnect', 'true')
        persist.type = kargs.get('type', 'PERSIST_SOURCE_IP')
        persist.netns_uuid = netns_uuid
        if persist.type is 'PERSIST_SOURCE_IP':
            persist.maskv4 = kargs.get('maskv4', '255.255.255.255')
            persist.maskv6 = kargs.get('maskv6', 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff')
            persist.sourceipTimeout = kargs.get('sourceipTimeout', DEFAULT_TIMEOUT)
        else:
            persist.cookieType = kargs.get('cookieType', 'PERSIST_COOKIE_INSERT')
            persist.cookieName = kargs.get('cookieName')
            persist.cookieDomain = kargs.get('cookieDomain', None) # 次参数必填
            persist.cookiePath = kargs.get('cookiePath', '/')
            persist.cookieTimeout = kargs.get('cookieTimeout', 0)
        return persist

    @log_helpers.log_method_call
    def createPersist(self, name, kargs={}, netns_uuid = '0'):
        '''
        创建会话保持
        '''
        try:
            persist = self.__buildPersist(name,kargs,netns_uuid)
            ret = self.adapi.createPersist(persist)
            if ret.isFailed():
                info = self.adapi.getPersistInfo(name)
                if info.isFailed() == False:
                    return
                raise Exception('createPersist failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deletePersist(self, name):
        '''
        删除会话保持
        '''
        try:
            ret = self.adapi.delPersist(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deletePersist failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __buildMonitor(self, name, type='MONITOR_CONNECT_TCP', kargs={}, netns_uuid = None):
        monitor = MonitorInfo()
        monitor.name = name
        monitor.type = type
        monitor.interval = kargs.get('interval', 5)
        if monitor.interval == 0:
            monitor.interval = 1
        monitor.timeout = kargs.get('timeout', 2)
        if monitor.timeout == 0:
            monitor.timeout = 255
        monitor.tryout = kargs.get('tryout', 3)
        monitor.debug = kargs.get('debug', 'false')
        monitor.addr = kargs.get('addr', '*')
        monitor.netns_uuid = netns_uuid
        # MONITOR_ICMPV4/MONITOR_ICMPV6
        if monitor.type is 'MONITOR_ICMPV4' or \
           monitor.type is 'MONITOR_ICMPV6':
            monitor.isFirewall = kargs.get('isFirewall', 'false')
            return monitor
        # MONITOR_CONNECT_HTTP
        if monitor.type is 'MONITOR_CONNECT_HTTP' or monitor.type is 'MONITOR_CONNECT_HTTPS':
            monitor.port = kargs.get('port', 0)
            if kargs.has_key('respCode'):
                if kargs['respCode'].find('-') != -1:
                    (l,h) = kargs['respCode'].split('-')
                    low_code = int(l)
                    high_code = int(h) + 1
                    for code in range(low_code, high_code):
                        monitor.respCode.append(str(code))
                else:
                    respCodes = kargs['respCode'].split(',')
                    for code in respCodes:
                        monitor.respCode.append(str(code))
                    #for code in kargs.get('respCode'):
                    #    monitor.respCode.append(code)
                    #monitor.respCode.append(kargs['respCode'])
            else:
                monitor.respCode = ['200', '302']
            monitor.urlMsg = "%s %s HTTP/1.0" % (kargs.get("method", "GET"), kargs.get('urlMsg', '/'))
            monitor.recvInclude = urllib.quote("\\|".join(["\\<"+e+"\\>" for e in monitor.respCode]))
            return monitor
        # MONITOR_CONNECT_TCP/MONITOR_CONNECT_UDP
        monitor.port = kargs.get('port', 0)
        monitor.maxRecvBuf = int(kargs.get('maxRecvBuf', 2048))
        monitor.sendMsg = kargs.get('sendMsg', None)           #必填
        monitor.recvInclude = kargs.get('recvInclude', None)   #必填
        monitor.closeSendMsg = kargs.get('closeSendMsg', None) # 必填
        return monitor

    @log_helpers.log_method_call
    def createMonitor(self, name, type='MONITOR_CONNECT_TCP', kargs={}, netns_uuid = '0'):
        '''
        新建监视器
        '''
        try:
            monitor = self.__buildMonitor(name, type, kargs, netns_uuid)
            ret = self.adapi.createMonitorInfo(monitor)
            if ret.isFailed():
                info = self.adapi.getMonitorInfo(name)
                if info.isFailed() == False:
                    return
                raise Exception('createMonitor failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateMonitor(self, name, type='MONITOR_CONNECT_TCP', kargs={}):
        '''
        更新监视器
        '''
        try:
            monitor = self.__buildMonitor(name, type, kargs)
            monitor.type = None # 更新时，不需要类型
            ret = self.adapi.updateMonitorInfo(name, monitor)
            if ret.isFailed():
                raise Exception('updateMonitor failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteMonitor(self, name):
        '''
        删除监视器
        '''
        try:
            ret = self.adapi.delMonitorInfo(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteMonitor failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __buildHttpCond(self, rules = []):
        cond = HttpCond()
        for rule in rules:
            if rule['type'] == "URI":
                cond.uri_cond = StrMatch(rule['compare_type'], False, rule['value'])
                cond.uri_cond.negated = rule.get('invert', False)
            else:
                header = HeaderCond()
                header.field_name = rule['key']
                header.field_value = StrMatch(rule['compare_type'], False, rule['value'])
                header.field_value.negated = rule.get('invert', False)
                cond.header_cond.append(header)

        return cond

    def __buildProtoCond(self, rules = []):
        cond = ProtoCond()
        cond.http_cond = self.__buildHttpCond(rules)
        return cond

    def __buildSchedRule(self, name, kargs={}):
        rule = SchedRule()
        rule.name = name

        rule.src_addr_cond = [AddrElement(None, addr_type="ADDR_ALL")]
        rule.service = kargs['service']
        rule.enable = kargs['enable']
        rule.fail_action = "SCHED_FAIL_ACTION_CLS_BY_RST"
        if kargs.get("rule_action"):
            rule.rule_action = kargs.get('rule_action')
        if kargs.get("pool_name"):
            rule.pool_name = kargs.get("pool_name")
        if kargs.get("redir_url"):
            rule.redir_url = kargs.get("redir_url")
        if kargs.get("netns_uuid"):
            rule.netns_uuid = kargs['netns_uuid']

        rule.proto_cond = self.__buildProtoCond(kargs.get("rules", []))
        return rule

    @log_helpers.log_method_call
    def createSchedRule(self, name, kargs={}):
        try:
            rule = self.__buildSchedRule(name, kargs)
            ret = self.adapi.createSchedRule(rule)
            if ret.isFailed():
                info = self.adapi.getSchedRule(name)
                if info.isFailed() == False:
                    return
                raise Exception('createSched failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateSchedRule(self, name, kargs={}):
        try:
            rule = self.__buildSchedRule(name, kargs)
            ret = self.adapi.updateSchedRule(name, rule)
            if ret.isFailed():
                raise Exception('updateSchedRule failed[%d, %s]' % (ret.getErrCode(),
                                                               ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteSchedRule(self, name):
        try:
            ret = self.adapi.deleteSchedRule(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteSchedRule failed[%d, %s]' % (ret.getErrCode(),
                                                               ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __buildPool(self, name, kargs={}, netns_uuid = None):
        nodepool = PoolInfo()
        nodepool.name = name
        #nodepool.monitors.append(kargs.get('monitor', 'ping'))
        nodepool.minMonitors = kargs.get('minMonitors', 0)
        nodepool.recoverInterval = kargs.get('recoverInterval', 0)
        nodepool.stepperInterval = kargs.get('stepperInterval', 0)
        nodepool.connStatAll = kargs.get('connStatAll', 'true')
        nodepool.persist1Name = kargs.get('persist1Name', 'none')
        nodepool.persist2Name = kargs.get('persist2Name', 'none')
        nodepool.schedMethod = kargs.get('schedMethod', 'SCHED_METHOD_FAIELD')
        nodepool.netns_uuid = netns_uuid
        if nodepool.schedMethod is 'SCHED_METHOD_QUEUE':
            nodepool.queueLength = kargs.get('queueLength', 100)
            nodepool.queueTimeout = kargs.get('queueTimeout', 5)
        nodepool.lbMethod = kargs.get('lbMethod', 'NODE_LB_RR')
        if nodepool.lbMethod is 'NODE_LB_HASH':
            nodepool.hashType = kargs.get('hashType', 'HASH_SRC_IP')
        nodepool.netnsId = kargs.get('netns_id', 0)
        return nodepool

    def __buildUpdatePool(self, name, kargs={}):
        nodepool = PoolInfo()
        nodepool.name = kargs.get('name', name)
        if 'persist1Name' in kargs:
            nodepool.persist1Name = kargs.get('persist1Name', 'none')
        if 'lbMethod' in kargs:
            nodepool.lbMethod = kargs.get('lbMethod', 'NODE_LB_RR')
            if nodepool.lbMethod is 'NODE_LB_HASH':
                nodepool.hashType = kargs.get('hashType', 'HASH_SRC_IP')

        return nodepool


    @log_helpers.log_method_call
    def createPool(self, name, kargs={}, netns_uuid = '0'):
        '''
        新建节点池
        '''
        try:
            nodepool = self.__buildPool(name, kargs, netns_uuid)
            ret = self.adapi.createPoolInfo(nodepool)
            if ret.isFailed():
                info = self.adapi.getPoolInfo(name)
                if info.isFailed() == False:
                    return
                raise Exception('createPool failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updatePool(self, name, kargs={}):
        '''
        更新节点池
        '''
        try:
            nodepool = self.__buildUpdatePool(name, kargs)
            ret = self.adapi.updatePoolInfo(name, nodepool)
            if ret.isFailed():
                raise Exception('updatePool failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deletePool(self, name):
        '''
        删除节点池
        '''
        try:
            ret = self.adapi.delPoolInfo(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deletePool failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __buildNode(self, name, ip="", port=80, kargs={}):
        '''
        构建AD 节点配置块
        @param name: 名字
        @param ip:  ip
        @param port:
        @param kargs:
        @return:  节点配置块
        '''
        node = NodeInfo()
        node.confStatus = 'NODE_ENABLE'
        if not kargs.get('enable', True):
            node.confStatus = 'NODE_DISABLE'
        # 节点权值校正
        node.ratio = int(kargs.get('ratio', 1))
        if node.ratio > DEFAULT_WEIGHT_MAX:
            node.ratio = DEFAULT_WEIGHT_MAX
            LOG.info("addriver member weight must be adjust ,"
                     "effective from 1 to DEFAULT_WEIGHT_MAX = 100 to ad")
        if node.ratio == 0:
            node.confStatus = 'NODE_DISABLE' #平滑退出
            node.ratio = 1

        node.maxConnects = kargs.get('maxConnects', 0)
        node.newConnects = kargs.get('newConnects', 0)
        node.maxRequest = kargs.get('maxRequest', 0)
        node.nodeIP = ip
        node.nodePort = port

        return node

    @log_helpers.log_method_call
    def createMember(self, name, ip="", port=80, kargs={}):
        '''
        往节点池添加解决点
        '''
        try:
            node = self.__buildNode(name, ip, port, kargs)
            ret = self.adapi.createNode(name, node)
            if ret.isFailed():
                info = self.adapi.getNodeInfo(name, ip, port)
                if info.isFailed() == False:
                    return
                raise Exception('createMember failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteMember(self, name, ip="", port=80, kargs={}):
        '''
        删除节点池中的节点
        '''
        try:
            ret = self.adapi.delNode(name, ip, port)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteMember failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateMember(self, name, ip="", port=80, kargs={}):
        '''
        更新节点信息
        '''
        try:
            node = self.__buildNode(name, ip, port, kargs)
            ret = self.adapi.updateNode(name, ip, port, node)
            if ret.isFailed():
                raise Exception('updateMemeber failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def getMemberStatus(self, pool):
        try:
            status = self.adapi.getNodeStatus(pool)
            if status.isFailed():
                LOG.error("Get Member Status For %s Failed: [%d, %s]" % (
                    pool, status.getErrCode(), status.getErrStr()
                ))
            return status
        except Exception as e:
            LOG.error("Traceback: %s, %s" % (e.message, traceback.format_exc()))

    @log_helpers.log_method_call
    def getAllMemberStatus(self):
        try:
            status = self.adapi.getAllNodeStatus()
            if status.isFailed():
                LOG.error("Get All Member Status Failed: [%d, %s]" % (
                    status.getErrCode(), status.getErrStr()
                ))
            return status
        except Exception as e:
            LOG.error("Traceback: %s, %s" % (e.message, traceback.format_exc()))

    @log_helpers.log_method_call
    def addPoolMonitor(self, poolname, monitorname):
        '''
        往节点池中添加监视器
        '''
        try:
            poolinfo = PoolInfo()
            poolinfo.monitors.append(monitorname)
            ret = self.adapi.updatePoolInfo(poolname, poolinfo)
            if ret.isFailed():
                raise Exception('addPoolMonitor failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def delPoolMonitor(self, poolname, monitorname):
        '''
        删除节点池中的监视器
        '''
        try:
            ret = self.adapi.delMonitor(poolname, monitorname)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('delPoolMonitor failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __buildVS(self, name, kargs={}, netns_uuid = None):
        '''
        构建AD 虚拟服务配置块
        @param name:
        @param kargs:
        @return:
        '''
        vs = VSInfo()
        vs.name = name
        vs.enable = kargs.get('enable', 'true')
        vs.ipgName = kargs.get('ipgName')           # 必填
        vs.serviceName = kargs.get('serviceName')   # 必填
        vs.poolName = kargs.get('poolName')         # 必填
        # 判断是否开启自动snat
        vs.autoSnat = kargs.get('autoSnat', 'true')
        # openstack HTTP  --> AD HTTP 7层负载
        # openstack HTTPS --> AD TCP 4层负载
        # openstack TCP   --> AD TCP 4层负载
        # openstack TERMINATED_HTTPS --> AD TCP 4层负载
        vs.mode = kargs.get('mode', 'VS_MODE_L4')
        vs.netns_uuid = netns_uuid
        if vs.mode is 'VS_MODE_L7':
            vs.httpSchedMode = kargs.get('httpSchedMode', 'HTTP_SCHED_MODE_EVERY_REQ') # 只用于7层VS
            vs.tcpCacheStream = kargs.get('tcpCacheStream', 'false') # 7层VS,服务为smtp/imap/pop时才填
            if vs.tcpCacheStream is 'true':
                vs.endStr = kargs.get('endStr', '/')
        if vs.mode is 'VS_MODE_L3':
            vs.serviceName = kargs.get('serviceName', 'layer3')
        vs.dnatEnable = kargs.get('dnatEnable', 'false')
        vs.netnsId = kargs.get('netns_id', 0)
        vs.limit = kargs.get('limit', None)
        vs.x_forward_enable = kargs.get('x_forward_enable', None)
        vs.sslProfile = kargs.get('ssl_profile', None)
        return vs

    def getPoolList(self):
        return self.adapi.getPoolList()

    @log_helpers.log_method_call
    def createVS(self, name, kargs={}, netns_uuid = '0'):
        '''
        新建虚拟服务
        @param name:  虚拟服务名字
        @param kargs:  携带配置字典
        @return:
        '''
        try:
            vs = self.__buildVS(name, kargs, netns_uuid)
            ret = self.adapi.createVS(vs)
            if ret.isFailed():
                info = self.adapi.getVSInfo(name)
                if info.isFailed() == False:
                    return
                raise Exception('createVS failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __buildUpdateVS(self, kargs={}):
        vs = VSInfo()
        vs.enable = kargs.get('enable', 'true')
        vs.ipgName = kargs.get('ipgName')
        vs.serviceName = kargs.get('serviceName')
        vs.poolName = kargs.get('poolName')
        vs.autoSnat = kargs.get('autoSnat')
        vs.httpSchedMode = kargs.get('httpSchedMode')
        vs.tcpCacheStream = kargs.get('tcpCacheStream')
        vs.endStr = kargs.get('endStr')
        vs.dnatEnable = kargs.get('dnatEnable')
        vs.limit = kargs.get('limit', None)
        vs.preProfile = kargs.get('preProfile', [])
        vs.sslProfile = kargs.get('ssl_profile', None)
        return vs
    @log_helpers.log_method_call
    def updateVS(self, name, kargs={}):
        '''
        更新虚拟服务
        @param name:  虚拟服务名字
        @param kargs:
        @return:
        '''
        try:
            vs = self.__buildUpdateVS(kargs)
            ret = self.adapi.updateVS(name, vs)
            if ret.isFailed():
                raise Exception('updateVS failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteVsSched(self, lb_name, sched_name):
        try:
            ret = self.adapi.deleteVsSched(lb_name, sched_name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteVsSched failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteVS(self, name):
        '''
        删除虚拟服务
        '''
        try:
            ret = self.adapi.delVS(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteVS failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def getACL(self, name):
        return self.adapi.getAclInfo(name)


    def __buildACL(self, name, kargs={}):
        '''
        构建AD 高级acl配置块
        @param name:
        @param kargs:
        @return:
        '''
        acl = AclInfo()
        acl.name = name
        acl.vsname = kargs.get('vsname')
        acl.enable = kargs.get('enable', 'true')
        acl.type = kargs.get('type', 'ADDR_ALL')
        acl.info = kargs.get('info')
        acl.connectLimit = kargs.get('connectLimit', 2000)
        return acl

    @log_helpers.log_method_call
    def createACL(self, name, kargs):
        '''
        创建高级ACL
        '''
        try:
            acl = self.__buildACL(name, kargs)
            ret = self.adapi.createAcl(acl)
            if ret.isFailed():
                info = self.adapi.getAclInfo(name)
                if info.isFailed() == False or info.getErrCode() != 105:
                    return
                raise Exception('createACL failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateACL(self, name, kargs={}):
        '''
        更新高级ACL
        '''
        try:
            acl = self.__buildACL(name, kargs)
            ret = self.adapi.updateAcl(acl)
            if ret.isFailed():
                raise Exception('updateACL failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteACL(self, name):
        '''
        删除高级ACL
        '''
        try:
            ret = self.adapi.deleteAcl(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteACL failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __buildWan(self, name, ifname, ips=[], args={}, netns_uuid=None):
        wan = WanInfo()
        wan.netns_uuid = netns_uuid
        wan.name = name
        wan.ifname = ifname
        wan.enable = 'true'
        dev_id = args.get('info_id', 1)
        for item in ips:
            ip = item + '/' + str(args.get('mask', 22))
            wan.ip.append(AddrElement(ip, dev_id))

        gw = args.get('gatewayIP')
        if gw == None:
            gw = '0.0.0.0'
        wan.gwAddr.append(AddrElement(gw))

        wan.maxUpBandwidth = 10000000
        wan.maxDownBandwidth = 10000000
        wan.upBusyRate = 80
        wan.downBusyRate = 80
        wan.enableDetect = 'false'
        wan.enableNicCheck = 'true'
        return wan

    def __addIPWithName(self, name, ips=[]):
        '''
        当存在WAN口配置时，只需要往网口中添加IP
        @return True: 网口存在，False：网口不存在
        '''
        wan = self.adapi.getWanInfo(name)
        if wan.isFailed() and self.__configExist(wan.getErrCode()) == False:
            return False
        # 网口肯定会有ip
        mask = wan.ip[0].ip.split('/')[1]
        wanInfo = WanInfo()
        wanInfo.ip = wan.ip
        ipSet = set(ip.ip for ip in wan.ip)
        flag = False
        for item in ips:
            ip = item + '/' + str(mask)
            if ip not in ipSet:
                wanInfo.ip.append(AddrElement(ip))
                flag = True
        if flag == True:
            self.adapi.updateWanInfo(name, wanInfo)
        return True


    @log_helpers.log_method_call
    def createWan(self, name, ifname, ips=[], args={}, netns_uuid='0'):
        '''
        创建wan口
        '''
        try:
            wan = self.adapi.getWanInfo(name)
            if not wan.isFailed():
                return True
            # 网口不存在，需要创建
            wan = self.__buildWan(name, ifname, ips, args, netns_uuid)
            ret = self.adapi.createWanInfo(wan)
            if ret.isFailed():
                raise Exception('createWan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def updateWan(self, name, ifname, ips=[], args={}):
        '''
        创建wan口
        '''
        try:
            # 网口不存在，需要创建
            wan = self.__buildWan(name, ifname, ips, args)
            ret = self.adapi.updateWanInfo(name, wan)
            if ret.isFailed():
                raise Exception('updateWan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteWan(self, name):
        '''
        删除wan口
        '''
        try:
            ret = self.adapi.deleteWanInfo(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteWan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __addIP(self, name, ips=[]):
        wan = self.adapi.getWanInfo(name)
        if wan.isFailed() and self.__configExist(wan.getErrCode()):
            raise Exception('getWanInfo failed[%d, %s]' % (wan.getErrCode(),
                                                          wan.getErrStr()))
        # 网口肯定会有ip
        mask = wan.ip[0].ip.split('/')[1]
        wanInfo = WanInfo()
        wanInfo.ip = wan.ip
        for item in ips:
            ip = item + '/' + str(mask)
            wanInfo.ip.append(AddrElement(ip))
        return wanInfo

    @log_helpers.log_method_call
    def addWanIP(self, name, ips=[]):
        '''
        添加WAN口IP
        '''
        try:
            # 先获取WAN口IP
            wan = self.__addIP(name, ips)
            self.adapi.updateWanInfo(name, wan)
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __findIP(self, ip, ips=[]):
        for item in ips:
            if item == ip:
                return True
        return False

    def __delIP(self, name, ips=[]):
        wan = self.adapi.getWanInfo(name)
        if wan.isFailed() and self.__configExist(wan.getErrCode()):
            raise Exception('getWanInfo failed[%d, %s]' % (wan.getErrCode(),
                                                          wan.getErrStr()))
        # 网口肯定会有ip
        mask = wan.ip[0].ip.split('/')[1]
        wanInfo = WanInfo()
        # 过滤掉要删除的ip
        for item in wan.ip:
            if self.__findIP(item.ip.split('/')[0], ips) == False:
                wanInfo.ip.append(item)
        return wanInfo

    @log_helpers.log_method_call
    def delWanIP(self, name, ips=[]):
        '''
        删除WAN口IP
        '''
        try:
            # 先获取WAN口IP
            wan = self.__delIP(name, ips)
            self.adapi.updateWanInfo(name, wan)
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def getWanInfo(self, name):
        try:
            return self.adapi.getWanInfo(name)
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def createLan(self, name, ifname, ips=[], args={}, netns_uuid='0'):
        '''
        创建lan口
        '''
        try:
            # 如果网口存在则直接添加IP
            info = self.adapi.getLanInfo(name)
            if info.isFailed() == False:
                return
            # 网口不存在，需要创建
            lan = self.__buildLan(name, ifname, ips, args, netns_uuid)
            ret = self.adapi.createLanInfo(lan)
            if ret.isFailed():
                raise Exception('createLan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteLan(self, name):
        '''
        删除lan口
        '''
        try:
            ret = self.adapi.deleteLanInfo(name)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteWan failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    def __buildLan(self, name, ifname, ips=[], args={}, netns_uuid=None):
        lan = LanInfo()
        lan.name = name
        lan.netns_uuid = netns_uuid
        lan.ifname = ifname
        lan.enable = 'true'
        dev_id = args.get('info_id', 1)
        for item in ips:
            ip = item + '/' + str(args.get('mask', 22))
            lan.ip.append(AddrElement(ip, dev_id))
        lan.enableDetect = 'false'
        lan.enableNicCheck = 'true'
        return lan

    @log_helpers.log_method_call
    def __buildRouteInfo(self, net, netns_id, gw, netns_uuid=None):
        route = StaticRouteInfo()
        route.netns_uuid = netns_uuid
        try:
            net_addr = net.split('/')[0]
            net_mask = int(net.split('/')[1])
        except Exception as e:
            LOG.error("net split fail net is %s"% net)
            raise Exception(e)

        route.net = net_addr
        route.mask = net_mask
        route.dev_id = 0
        route.netns_id = netns_id
        route.gw_addr = gw
        return route

    @log_helpers.log_method_call
    def createRoute(self, net, netns_id, gw, netns_uuid='0'):
        """创建route"""
        route = self.__buildRouteInfo(net, netns_id, gw, netns_uuid)
        try:
            ret = self.adapi.createStaticRoute(route)
            if ret.isFailed():
                # 创建失败时，查看一下netns是否已经存在，存在则更新
                result = self.adapi.getStaticRouteInfo(route.net, route.mask, route.netns_id)
                if result.isFailed() == False:
                    return

                raise Exception('createRoute failed[%d, %s]' % (ret.getErrCode(),
                                                              ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def deleteRoute(self, net, netns_id):
        '''
        删除route配置
        '''
        try:
            ip = net.split('/')[0]
            mask = int(net.split('/')[1])
            ret = self.adapi.delStaticRoute(ip, mask, netns_id)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteStaticRoute failed[%d, %s]'\
                        % (ret.getErrCode(), ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)
        
    @log_helpers.log_method_call
    def clearRouteInNetns(self, netns_id):
        '''
        删除route配置
        '''
        try:
            ret = self.adapi.delStaticRoute('*', 0, netns_id)
            if ret.isFailed() and self.__configExist(ret.getErrCode()):
                raise Exception('deleteStaticRoute failed[%d, %s]'\
                        % (ret.getErrCode(), ret.getErrStr()))
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            raise Exception(e)
        

if __name__ == '__main__':
    ad = ADHOST('cluster', 'root1234', '200.200.146.121', '443')

    #ad.createRoute('0.0.0.0/0', 21, '124.126.127.1')
    #ad.deleteRoute('0.0.0.0/0', 21)
    #print ad.deleteVlan('op-vlan-52db8248-76de-4585-b6e0-e7999400ee0d')

    '''
    ad.createVlan('openstack_test1_vlan', 101, 'bond0')
    ad.createVlan('openstack_test2_vlan', 102, 'bond0')
    vlan_device1 = ad.getVlanDevice('openstack_test1_vlan')
    vlan_device2 = ad.getVlanDevice('openstack_test2_vlan')
    ad.createNetns('openstack_test1', 'test_user1')
    ad.createNetns('openstack_test2', 'test_user1')
    netns_info1 = ad.getNetnsInfo('openstack_test1')
    netns_info2 = ad.getNetnsInfo('openstack_test2')
    ad.createMacvlan('openstack_test1', '0A:ff:fe:22:33:44', vlan_device1, netns_info1['netnsId'])
    ad.createMacvlan('openstack_test2', '0A:ff:fe:22:33:22', vlan_device2, netns_info2['netnsId'])
    macvlan_device1 = ad.getMacvlanDevice('openstack_test1')
    macvlan_device2 = ad.getMacvlanDevice('openstack_test2')
    macvlan_device_list = [macvlan_device1, macvlan_device2]
    netns_info_list = [netns_info1, netns_info2]
    for i in range(0,2):
        tmp_name = 'openstack_test%d'%(i+1)
        ad.createIPGroup(tmp_name, ['192.168.1.100'],\
                {'mask':20, \
                'gatewayIP':'192.168.1.1', 'wan':tmp_name,
                'ifname':macvlan_device_list[i], 'info_id':netns_info_list[i]['appgId']})
        ad.createServ(tmp_name, 'SRC_TCP', [8080])
        ad.createPool(tmp_name, {'netns_id':netns_info_list[i]['netnsId'],'monitor':'ping'})
        ad.createMember(tmp_name, '192.168.1.10', 0, {'ratio':1})
        ad.createMember(tmp_name, '192.168.1.11', 0, {'ratio':1})
        ad.createMember(tmp_name, '192.168.1.12', 0, {'ratio':1})
        ad.createVS(tmp_name, {'poolName': tmp_name, "ipgName":tmp_name,\
                'serviceName':tmp_name, 'enable':'true', 'mode':'VS_MODE_L4',\
                'netns_id':netns_info_list[i]['netnsId']})
        tmp_args={
                'interval':5,
                'timeout':3,
                'tryout':3
            }
        ad.createMonitor(tmp_name, 'MONITOR_ICMPV4', tmp_args)
        ad.addPoolMonitor(tmp_name, tmp_name)

    
    #clearRouteInNetns()
    #ad.deleteRoute('34.22.44.0/24', 0)
    #ad.deletePool('testpool')
    #ad.createPool('testpool')
    #ad.deleteMacvlan('test123')
    #ad.deleteVlan('op-vlan-52db8248-76de-4585-b6e0-e7999400ee0d')
    #ad.deleteVS('test1')

    #ad.createNetns('openstack_test5', 'test_user1')
    #ad.deleteNetns('openstack_test5')

    ad.createVlan('openstack_2_123', 123, 'eth2')
    vlan_device = ad.getVlanDevice('openstack_2_123')
    ad.createNetns('openstack_test5', 'test_user1')
    netns_id = ad.getNetnsId('openstack_test5')
    ad.createMacvlan('openstack_test5', '0A:ff:fe:22:33:44', vlan_device, netns_id)
    #ad.deleteMacvlan('openstack_test5')
    #print ad.getMacvlanDevice('openstack_test2')
    '''

    #wan test
    #args = {'mask':24,'gatewayIP':'43.23.111.2','info_id':8}
    #ips = ['43.23.111.33', '22.44.123.44']
    #ad.createWan('wan2', 'eth2', ips, args)
    #ad.deleteWan('wan5')
