# -*- coding: utf-8 -*-
# Copyright (c) 2008-2020, AD SANGFOR
# Filename: SchedRule.py
#

from ErrorInfo import ErrInfo
from AddrElement import AddrElement

class StrMatch(object):
    def __init__(self, type=None, case=None, patt=None):
        self.type = type
        self.case_sensitive = case
        self.negated = None
        self.pattern = patt
        self.hexadecimal = None
    @classmethod
    def toDict(cls, match):
        mDict = {}
        if match.type is not None:
            mDict['type'] = match.type
        if match.case_sensitive is not None:
            mDict['case_sensitive'] = match.case_sensitive
        if match.negated is not None:
            mDict['negated'] = match.negated
        if match.pattern is not None:
            mDict['pattern'] = match.pattern
        if match.hexadecimal is not None:
            mDict['hexadecimal'] = match.hexadecimal
        return mDict

class HeaderCond(object):
    def __init__(self):
        self.filed_name = None
        self.field_value = None
    @classmethod
    def toDict(cls, cond):
        cDict = {}
        if cond.field_name is not None:
            cDict['field_name'] = cond.field_name
        if cond.field_value is not None:
            cDict['field_value'] = StrMatch.toDict(cond.field_value)
        return cDict

class HttpCond(object):
    def __init__(self):
        self.version_cond = None
        self.method_cond = None
        self.uri_cond = None
        self.header_cond = []
        self.rsp_header_cond = []
        self.status_code = None
        self.rsp_version_cond = None

    @classmethod
    def toDict(cls, cond):
        cDict = {}
        if cond.version_cond is not None:
            cDict['version_cond'] = cond.version_cond
        if cond.method_cond is not None:
            cDict['method_cond'] = cond.method_cond
        if cond.uri_cond is not None:
            cDict['uri_cond'] = StrMatch.toDict(cond.uri_cond)
        cDict['header_cond'] = []
        for h in cond.header_cond:
            cDict['header_cond'].append(HeaderCond.toDict(h))
        cDict['rsp_header_cond'] = []
        for h in cond.rsp_header_cond:
            cDict['rsp_header_cond'].append(HeaderCond.toDict(h))
        if cond.status_code is not None:
            cDict['status_code'] = StrMatch.toDict(cond.status_code)
        if cond.rsp_version_cond is not None:
            cDict['rsp_version_cond'] = cond.rsp_version_cond
        return cDict

class ProtoCond(object):
    def __init__(self):
        self.http_cond = None
        pass
    @classmethod
    def toDict(cls, cond):
        cDict = {}
        if cond.http_cond is not None:
            cDict['http_cond'] = HttpCond.toDict(cond.http_cond)
        return cDict

class SchedRule(ErrInfo):
    def __init__(self, httpBody=None):
        ErrInfo.__init__(self, httpBody)
        self.name = None
        self.enable = None
        self.src_addr_cond = []
        self.service = None
        self.proto_cond = None
        self.pool_name = None
        self.fail_action = None
        self.rule_action = None
        self.close_method = None
        self.page_id = None
        self.request_rewrite_rule = []
        self.reply_rewrite_rule = []
        self.dst_addr_cond = []
        self.netns_uuid = None
        self.redir_url = None

        if httpBody == "" or httpBody is None:
            self.originStr = ""
            return
        if self.isFailed() or self.getErrCode() == 0:
            return
        if not isinstance(httpBody, unicode):
                httpBody = unicode(httpBody, "utf8")
        self.originStr = httpBody

    @classmethod
    def toDict(cls, rule):
        rDict = {}
        if rule.name is not None:
            rDict['name'] = rule.name
            rDict['enable'] = True if rule.enable else False

        rDict['src_addr_cond'] = []
        for cond in rule.src_addr_cond:
            rDict['src_addr_cond'].append(AddrElement.toDict(cond))

        if rule.enable is not None:
            rDict['enable'] = rule.enable

        if rule.service is not None:
            rDict['service'] = rule.service

        if rule.proto_cond is not None:
            rDict['proto_cond'] = ProtoCond.toDict(rule.proto_cond)

        if rule.pool_name is not None:
            rDict['pool_name'] = rule.pool_name

        if rule.fail_action is not None:
            rDict['fail_action'] = rule.fail_action

        if rule.rule_action is not None:
            rDict['rule_action'] = rule.rule_action

        if rule.close_method is not None:
            rDict['close_method'] = rule.close_method

        if rule.page_id is not None:
            rDict['page_id'] = rule.page_id

        rDict['dst_addr_cond'] = []
        for cond in rule.dst_addr_cond:
            rDict['dst_addr_cond'].append(AddrElement.toDict(cond))

        if rule.netns_uuid is not None:
            rDict['netns_uuid'] = rule.netns_uuid

        if rule.redir_url is not None:
            rDict['redir_url'] = rule.redir_url

        return rDict

