# -*- coding: utf-8 -*-
# Copyright (c) 2008-2020, AD SANGFOR
# Filename: VxlanInfo.py
# 定义vxlan信息
from Configuration import Configuration
from APIException import APIException
from ErrorInfo import ErrInfo
from http.HttpClient import HttpClient
from ParentList import ParentList
from urllib2 import base64
import JSONObj as jsono
import urllib
import json
import codecs


class VxlanInfo(ErrInfo):
    '''
    vxlan信息
    {
        'id': ,
        'name': ,
        'vxlan_id': ,
        'device':,
        'vxlan_domain':,
    }
    '''
    def __init__(self, httpBody=None):
        ErrInfo.__init__(self, httpBody)
        self.name = None        # vxlan名称
        self.vxlan_id = 0        # vxlan id
        self.device = None      # 设备名
        self.vxlan_domain = 0   # vxlan域
        if httpBody == "" or httpBody is None:
            self.originStr = ""
            return
        if self.isFailed() or self.getErrCode() == 0:
            return
        if not isinstance(httpBody, unicode):
            httpBody = unicode(httpBody, "utf8")
        self.originStr = httpBody
        try:
            self.jsonObj = jsono.JSONObj(httpBody)
            if self.jsonObj.hasTag('VxlanInfoType'):
                jsonVxlanInfo = self.jsonObj['VxlanInfoType']
                if 'name' in jsonVxlanInfo:
                    self.name = urllib.unquote(
                        base64.b64decode(jsonVxlanInfo['name']))
                if 'device' in jsonVxlanInfo:
                    self.device = urllib.unquote(
                        base64.b64decode(jsonVxlanInfo['device']))
                if 'vxlan_id' in jsonVxlanInfo:
                    self.vxlan_id = int(jsonVxlanInfo['vxlan_id'])
                if 'vxlan_domain' in jsonVxlanInfo:
                    self.vxlan_domain = int(jsonVxlanInfo['vxlan_domain'])
        except jsono.JSONObjException, e:
            raise APIException(3, e.reason)

    @classmethod
    def vxlanInfoToDict(cls, vxlanInfo):
        vxlanDict = {}
        keys = {'name', 'vxlan_id', 'device', 'vxlan_domain'}
        for key in keys:
            if getattr(vxlanInfo, key, None) is not None:
                vxlanDict[key] = getattr(vxlanInfo, key, None)
        return vxlanDict

    @classmethod
    def generatingVxlanInfo(cls, httpBody):
        return VxlanInfo(httpBody)

    @classmethod
    def generatingVxlanList(cls, httpBody):
        return VxlanList(httpBody)


class VxlanList(ParentList):
    '''
    vxlan信息列表
    {
        'data': [
            {
                'id': ,
                ......
            }
        ]
    }
    '''
    def __init__(self, httpBody):
        ParentList.__init__(self, httpBody)
        if httpBody == "" or httpBody is None:
            self.originStr = ""
            return
        if self.isFailed() or self.getErrCode() == 0:
            return
        if not isinstance(httpBody, unicode):
            httpBody = unicode(httpBody, "utf8")
        self.originStr = httpBody
        try:
            self.jsonObj = jsono.JSONObj(httpBody)
            if not self.jsonObj.hasTag('data'):
                return
            for item in self.jsonObj['data']:
                vxlan = VxlanInfo(json.dumps(item, ensure_ascii=False))
                self.elements.append(vxlan)
        except jsono.JSONObjException, e:
            raise APIException(3, e.reason)
