# -*- coding: utf-8 -*-
# Copyright (c) 2008-2020, AD SANGFOR
# Filename: VxlanInfo.py
# 定义vxlan信息
from Configuration import Configuration
from APIException import APIException
from ErrorInfo import ErrInfo
from http.HttpClient import HttpClient
from ParentList import ParentList
from urllib2 import base64
import JSONObj as jsono
import urllib
import json
import codecs


class VxnetInfo(ErrInfo):
    '''
    vxnet信息
    {
        'id': ,
        'name': ,
        'vxnet_id': ,
        'device':,
        'vxnet_domain':,
    }
    '''
    def __init__(self, httpBody=None):
        ErrInfo.__init__(self, httpBody)
        self.name = None        # vxnet名称
        self.vxlan_domain = 0   # vxnet域
        self.vteps = []
        self.port = 0
        if httpBody == "" or httpBody is None:
            self.originStr = ""
            return
        if self.isFailed() or self.getErrCode() == 0:
            return
        if not isinstance(httpBody, unicode):
            httpBody = unicode(httpBody, "utf8")
        self.originStr = httpBody
        try:
            self.jsonObj = jsono.JSONObj(httpBody)
            if self.jsonObj.hasTag('VxnetInfoType'):
                jsonVxnetInfo = self.jsonObj['VxnetInfoType']
                if 'name' in jsonVxnetInfo:
                    self.name = urllib.unquote(
                        base64.b64decode(jsonVxnetInfo['name']))
                if 'vteps' in jsonVxnetInfo:
                    self.vteps = jsonVxnetInfo['vteps']
                if 'vxlan_domain' in jsonVxnetInfo:
                    self.vxlan_domain = int(jsonVxnetInfo['vxlan_domain'])
                if 'port' in jsonVxnetInfo:
                    self.port = int(jsonVxnetInfo['port'])
        except jsono.JSONObjException, e:
            raise APIException(3, e.reason)

    @classmethod
    def vxnetInfoToDict(cls, vxnetInfo):
        vxnetDict = {}
        keys = {'name', 'vteps', 'port', 'vxlan_domain'}
        for key in keys:
            if getattr(vxnetInfo, key, None) is not None:
                vxnetDict[key] = getattr(vxnetInfo, key, None)
        return vxnetDict

    @classmethod
    def generatingVxnetInfo(cls, httpBody):
        return VxnetInfo(httpBody)

    @classmethod
    def generatingVxnetList(cls, httpBody):
        return VxnetList(httpBody)


class VxnetList(ParentList):
    '''
    vxnet信息列表
    {
        'data': [
            {
                'id': ,
                ......
            }
        ]
    }
    '''
    def __init__(self, httpBody):
        ParentList.__init__(self, httpBody)
        if httpBody == "" or httpBody is None:
            self.originStr = ""
            return
        if self.isFailed() or self.getErrCode() == 0:
            return
        if not isinstance(httpBody, unicode):
            httpBody = unicode(httpBody, "utf8")
        self.originStr = httpBody
        try:
            self.jsonObj = jsono.JSONObj(httpBody)
            if not self.jsonObj.hasTag('data'):
                return
            for item in self.jsonObj['data']:
                vxnet = VxnetInfo(json.dumps(item, ensure_ascii=False))
                self.elements.append(vxnet)
        except jsono.JSONObjException, e:
            raise APIException(3, e.reason)
