# coding=utf-8

import datetime
import hashlib
import logging as std_logging
import urllib2
import uuid
import traceback

import sys
from eventlet import greenthread
from time import time
import commands

from neutron.common import constants as q_const
#from neutron.common.exceptions import InvalidConfigurationOption
#from neutron.common.exceptions import NeutronException
try:
    from neutron_lib import constants as plugin_const
except:
    from neutron.plugins.common import constants as plugin_const
from neutron_lbaas.services.loadbalancer import constants as lb_const
try:
    from neutron_lib.api.definitions import portbindings
except:
    from neutron.extensions import portbindings

from oslo_config import cfg
from oslo_log import helpers as log_helpers
from oslo_log import log as logging
from oslo_utils import importutils
from ad import ADHOST as AD

from sangfor_openstack_agent.lbaasv2.drivers.ad.lbaas_driver import LBaaSBaseDriver
from sangfor_openstack_agent.lbaasv2.drivers.ad import constants_v2 as sangforconst
from sangfor_openstack_agent.lbaasv2.drivers.ad.adV2.api.APIException import APIException

from proto_builder import ServiceBuilder, UuidBuilder

from netaddr.strategy import ipv4, ipv6

import re

'''
全局变量定义
'''
_DEFAULT_POOL = 'openstack-default-pool'
_MONITOR_RETRIES = 255
_MONITOR_MAX_DELAY = 255
_MONTTOR_MAX_TIMEOUT = 255
_MONITOR_TYPE_MAP = {
    'TCP':'MONITOR_CONNECT_TCP',
    'PING':'MONITOR_ICMPV4',
    'HTTP':'MONITOR_CONNECT_HTTP',
    'HTTPS':'MONITOR_CONNECT_HTTPS'
}

LOG = logging.getLogger(__name__)
__VERSION__ = '0.1.1'

'''
配置文件定义
'''
OPTS = [
    cfg.StrOpt(
        'server_host',
        default='200.200.144.196',
        help='connect to AD server_address'
    ),
    cfg.IntOpt(
        'server_port',
        default=443,
        help='connect to AD server_port',
    ),
    cfg.IntOpt(
        'vxlan_local_port',
        default=0,
        help='vxlan local port',
    ),

    cfg.IntOpt(
        'vxlan_remote_port',
        default=4789,
        help='vxlan remote port',
    ),
    cfg.StrOpt(
        'server_admin',
        default='admin',
        help='username to login'
    ),
    cfg.StrOpt(
        'server_password',
        default='root1234',
        help='password to login',
        secret=True
    ),

    cfg.ListOpt(
        'advertised_tunnel_types', default=[],
        help='tunnel types which are advertised to other VTEPs'
    ),
    cfg.ListOpt(
        'advertised_tunnel_ips', default=[],
        help='tunnel types which are advertised to other VTEPs'
    ),
    cfg.StrOpt(
        'advertised_tunnel_wan',
        default=None,
        help='password to login'
    ),
    cfg.ListOpt(
        'vad_manager_info_mapping', default=['eth1:200.200.144.113:22'],
        help='vad manager info'
    ),
    cfg.ListOpt(
        'external_physical_mappings', default=['default:eth1:True'],
        help='Mapping between Neutron physical_network to interfaces'
    ),

    cfg.ListOpt(
        'local_interface_ip', default=['eth1:200.200.1.2:24'],
        help='Mapping between Neutron physical_network to interfaces'
    ),

    cfg.BoolOpt(
        'snat_port_enable',
        default=True,
        help='create port for snat to member',
    ),

    cfg.IntOpt(
        'LB_concurrent_connections',
        default=0,
        help='concurrent connections',
    ),

    cfg.IntOpt(
        'LB_connections_per_second',
        default=0,
        help='connections per second',
    ),

    cfg.IntOpt(
        'LB_throughtputs_bytes',
        default=0,
        help='concurrent connections',
    ),

    cfg.BoolOpt(
        'x_forwarded_for_enable',
        default=False,
        help='concurrent connections',
    ),
]

'''
AD驱动类，直接调用AD api方法下发配置
'''
class ADDriver(LBaaSBaseDriver):

    def __init__(self, conf, registerOpts=True):

        super(ADDriver, self).__init__(conf)
        self.conf = conf

        self.method = {
                'members':'member',
                'healthmonitors':'health_monitor',
                'pools':'pool',
                'listeners':'listener',
                'loadbalancer':'loadbalancer',
                'l7policies':'l7policy',
                'l7rules':'l7rule'
        }

        if registerOpts:
            self.conf.register_opts(OPTS)

        if not ipv4.valid_str(self.conf.server_host):
            raise Exception("invalid server host %s" % self.conf.server_host)

        self.plugin_rpc = None

        #TODO:需要整理，对外配置显示，回作为agent属性存储到db中
        self.agent_configurations = {}
        self.agent_configurations["tunnel_types"] = self.conf.advertised_tunnel_types
        self.agent_configurations["tunneling_ips"] = self.conf.advertised_tunnel_ips
        self.agent_configurations['common_networks']={}

        self.username = self.conf.server_admin
        self.password = self.conf.server_password
        #改为通过AD API连接物理AD
        self.ad = AD(self.username, self.password, self.conf.server_host, self.conf.server_port)
        #没人操作这个loadbalancer，删掉吧
        #self.loadbalancer = {}
        self.interface_mapping={}
        self.local_interface_ip = {}
        self.tagging_mapping={}

        #对外物理接口，用来建立vlan或vxlan，这部分可以复用
        for maps in self.conf.external_physical_mappings:
            intmap = maps.split(':')
            net_key = str(intmap[0]).strip()
            self.interface_mapping[net_key] = str(intmap[1]).strip()
            self.tagging_mapping[net_key] = intmap[2].strip().lower() == 'true'
            LOG.debug('physical_network %s = interface %s, tagged %s'
                      % (net_key, intmap[1], intmap[2]))
        self.agent_configurations['bridge_mappings'] = \
            self.interface_mapping

        #应该是vxlan等tunnel引用的通信ip列表，可以不动
        for maps in self.conf.local_interface_ip:
            intmap = maps.split(':')
            net_key = str(intmap[0].strip())
            self.local_interface_ip[net_key]=[intmap[1].strip(), int(intmap[2].strip())]
            pass

        """
        self.reset = True
        """
        self.reset = False

    def convert_member_status(self, node_status):
        if node_status == "NODE_NORMAL":
            return lb_const.ONLINE
        else:
            return lb_const.OFFLINE

    def get_member_status(self, pool):
        try:
            nodes = self.ad.getMemberStatus(self._get_pool_name(pool))
            if nodes is None:
                return []
            status = []
            for n in nodes.elements:
                k = n.nodeIP + ':' + str(n.nodePort)
                s = self.convert_member_status(n.healStatus)
                status.append({'key':k, 'status':s})
            return status
        except:
            LOG.error("Traceback: %s" % traceback.format_exc())
            return []

    def get_all_member_status(self):
        try:
            nodes = self.ad.getAllMemberStatus()
            if nodes is None:
                return []
            status = []
            for n in nodes.elements:
                k = n.poolName+':'+self.netaddr_format(n.nodeIP)+':'+str(n.nodePort)
                s = self.convert_member_status(n.healStatus)
                status.append({'key':k, 'status':s})
            return status
        except:
            LOG.error("Traceback: %s" % traceback.format_exc())
            return []

    def _getnetwork_from_service(self, service, network_id):
        if 'networks' in service:
            return service['networks'][network_id]

    def _get_subnets_to_from_service(self, service, subnet_id):
        if 'subnets' in service:
            return service['subnets'][subnet_id]
    def set_tunnel_rpc(self, tunnel_rpc):
       self._tunnel_rpc = tunnel_rpc

    def set_l2pop_rpc(self, l2_pop_rpc):
       self._l2pop_rpc = l2_pop_rpc

    def get_tunnel_ips(self):
        if 'tunneling_ips' in self.agent_configurations:
            return self.agent_configurations['tunneling_ips']
        else:
            return []

    def notify_vtep_added(self, network, vtep_ip_address):
        if self.conf.l2_population and self._l2pop_rpc:
            LOG.debug("add_fdb_entry for %s" % vtep_ip_address)
            fdb_entries = {
                    network['id']: {
                        'ports': {
                            vtep_ip_address: [q_const.FLOODING_ENTRY]
                        },
                        'network_type': network['provider:network_type'],
                        'segment_id': network['provider:segmentation_id']
                        }
                    }
            self._l2pop_rpc.add_fdb_entries(self._context, fdb_entries)

    def notify_vtep_removed(self, network, vtep_ip_address):
        if self.conf.l2_population and self._l2pop_rpc:
            LOG.debug("delete_fdb_entry for %s" % vtep_ip_address)
            fdb_entries = {
                    network['id']: {
                        'ports': {
                            vtep_ip_address: [q_const.FLOODING_ENTRY]
                        },
                        'network_type': network['provider:network_type'],
                        'segment_id': network['provider:segmentation_id']
                        }
                    }
            self._l2pop_rpc.remove_fdb_entries(self._context, fdb_entries)

    def _get_subnets_to_assure(self, service):
        networks = dict()
        loadbalancer = service['loadbalancer']
        lb_status = loadbalancer['provisioning_status']

        if lb_status != plugin_const.PENDING_DELETE:
            if 'network_id' in loadbalancer:
                network = self._getnetwork_from_service(service, loadbalancer['network_id'])
                subnet = self._get_subnets_to_from_service(service, loadbalancer['vip_subnet_id'])
                networks[network['id']] = {'network': network,
                        'subnet': subnet,
                        'is_for_member': False}
                pass
            pass
        return networks.values()

    def set_plugin_rpc(self, plugin_rpc):
       self.plugin_rpc = plugin_rpc

    def _get_lb_appg_id(self, lb_id):
        '''
        TODO: 获取虚拟服务对应的appg id
        @param lb_id lb的id
        @return 返回appg_id(暂时返回默认组id 1)
        '''
        return 1

    def _assure_netns(self, lb):
        '''
        @创建netns
        @param service: 创建虚拟服务的完整信息
        @return 返回netns_info['tenant', 'netnsId', 'appgId', 'name']
        '''
        netns_name = self._get_loadbalancer_name(lb)
        #appg_id = self._get_lb_appg_id(lb['id'])
        #TODO: 需要查询tenant名称，这里先用id代替
        conns = self.conf.LB_concurrent_connections
        new_conns = self.conf.LB_connections_per_second
        flows = self.conf.LB_throughtputs_bytes
        self.ad.createNetns(netns_name, lb['tenant_id'], lb['id'], conns, new_conns, flows)
        #由于之前api只返回成功失败，这里还要获取一下netns_id信息
        netns_info = self.ad.getNetnsInfo(netns_name)
        return netns_info

    def _assure_macvlan(self, netns_id, ifname, macvlan_name, mac_addr, netns_uuid):
        '''
        @创建macvlan
        @param netns_id macvlan对应netns的id
        @param ifname   macvlan引用的if
        @return macvlan 对应device
        '''

        try:
            self.ad.createMacvlan(macvlan_name, mac_addr, ifname, netns_id, netns_uuid)
        except APIException as ex:
            LOG.error(u"Create Macvlan error: %s" % traceback.format_exc())
            pass

        #需要在创建成功之后获取Macvlan的device字段
        macvlan_device = self.ad.getMacvlanDevice(macvlan_name)
        return macvlan_device

    def _assure_network(self, network, service):
        if not network:
            LOG.error(' assure_network: '
                      'Attempted to assure a network with no id..skipping.')
            return

        if network['provider:network_type'] == 'vlan':
            return self._assure_network_vlan(network)
        elif network['provider:network_type'] == 'vxlan':
            return self._assure_network_vxlan(network, service)
        elif network['provider:network_type'] == 'flat':
            return ['flat', '']
        else:
            error_message = 'Unsupported network type %s.' \
                            % network['provider:network_type'] + \
                            ' Cannot setup network.'
            LOG.error(error_message)
            raise Exception(error_message)

    def _get_tunnel_name(self, network):
        tunnel_type = network['provider:network_type']
        tunnel_id = network['provider:segmentation_id']
        return 'tunnel-' + str(tunnel_type) + '-' + str(tunnel_id)

    def _get_vxlan_vteps(self, service):
        vxlan=[]
        if 'loadbalancer' in service and 'vxlan_vteps' in service['loadbalancer']:
            vxlan = service['loadbalancer']['vxlan_vteps']
        return vxlan

    def _assure_network_vxlan(self, network, service):
        '''
        @创建vxlan网口
        @param network 对应网络结构
        @return ['vxlan', vxlan_device] device为设备上vxlan设备的实际名称
        '''

        vxlanid = network['provider:segmentation_id']
        vxlan_name = self._get_vxlan_name(network['id'])
        try:
            self.ad.createVxlan(vxlan_name, vxlanid, 0)
        except APIException as ex:
            LOG.error(u"Create Vxlan error: %s" % traceback.format_exc())
            pass

        # 需要创建成功之后才能获取到具体的device字段
        vxlan_dev = self.ad.getVxlanDevice(vxlan_name)
        if vxlan_dev:
            vteps = self.get_tunnel_ips()
            for ip in vteps:
                self.notify_vtep_added(network, ip)
        return ['vxlan', vxlan_dev]

    def _get_interface(self, network):

        interface = self.interface_mapping['default']
        net_key = network['provider:physical_network']

        if net_key in self.interface_mapping:
            interface = self.interface_mapping[net_key]
        else:
            interface = self.interface_mapping['default']

        return interface

    def get_vlan_tuple(self, network):
        interface = self.interface_mapping['default']
        tagged = self.tagging_mapping['default']

        net_key = network['provider:physical_network']
        if net_key in self.interface_mapping:
            interface = self.interface_mapping[net_key]
            tagged = self.tagging_mapping[net_key]
        else:
            interface = self.interface_mapping['default']
            tagged = self.tagging_mapping['default']

        return interface, tagged

    def _get_vxlan_local_ip(self, network):
        return self.local_interface_ip[self._get_interface(network)][0]

    def _assure_network_vlan(self, network):
        '''
        @创建vlan网口
        @param network 对应网络结构
        @return ['vlan', vlan_device] device为设备上vlan设备的实际名称
        '''

        vlan_name = self._get_vlan_name(network['id'])

        interface, tagged = self.get_vlan_tuple(network)

        if tagged:
            vlanid = network['provider:segmentation_id']
        else:
            vlanid = 0

        try:
            self.ad.createVlan(vlan_name, vlanid, interface)
        except APIException as ex:
            LOG.error(u"Create Vlan error: %s" % traceback.format_exc())
            pass

        #需要创建成功之后才能获取到具体的device字段
        vlan_dev = self.ad.getVlanDevice(vlan_name)

        return ['vlan', vlan_dev]

    def set_context(self, context):
        self._context = context
        pass

    def post_init(self):
        pass

    def _get_vlan_name(self, net_id):
        '''
        @获取vlan口配置名称
        @param net_id network的uuid
        @return vlan名称
        '''
        return "op-vlan-" + str(net_id)

    def _get_vxlan_name(self, net_id):
        '''
        @获取vxlan口配置名称
        @param net_id network的uuid
        @return vxlan名称
        '''
        return "op-vxlan-" + str(net_id)

    def _get_loadbalancer_name(self, lb):
        '''
        @获取所有虚拟服务相关配置名称
        @param lb_id lb的uuid
        @return 配置名称
        '''

        return "openstack-" + str(lb['id'])

    def _get_listener_name(self, listener):
        '''
        @获取所有listener相关配置名称(服务 VS)
        @param listener_id listener的uuid
        @return 配置名称
        '''

        return "openstack-" + str(listener['id'])

    def _get_pool_name(self, pool):
        '''
        @获取所有pool相关配置名称(节点池 回话保持)
        @param pool_id pool的uuid
        @return 配置名称
        '''

        return "openstack-" + str(pool['id'])

    def _get_monitor_name(self, monitor):
        '''
        @获取所有monitor相关配置名称(节点池 回话保持)
        @param monitor_id monitor的uuid
        @return 配置名称
        '''

        return "openstack-" + str(monitor['id'])

    def create_loadbalancer(self, loadbalancer, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            LOG.error(u'create loadbalancer error %s' % traceback.format_exc())

    def update_loadbalancer(self, old_lb ,lb, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            LOG.error("update loadbalancer error: %s" % traceback.format_exc())
            return
        pass

    def delete_loadbalancer(self, lb, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            #service['loadbalancer']['provisioning_status'] = plugin_const.ERROR
            #直接删除，删除不失败
            self._update_service_status(service)
            LOG.error("delete loadbalancer failed: %s", traceback.format_exc())
        pass

    @log_helpers.log_method_call
    def _createDefaultPool(self, name):
        create = True
        pooList = self.ad.getPoolList()
        for pool in pooList:
            if pool.name == name:
                create = False
                break
        if create:
            self.ad.createPool(name)

    def _set_listener_status(self, listener, service, status):
        l = service['listeners'][listener['id']]

        if l['provisioning_status'] != status:
            l['provisioning_status'] = status
            LOG.debug("set listener status to %s" % status)

    def create_listener(self, listener, service):
        self._set_listener_status(listener, service, plugin_const.PENDING_CREATE)
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("createServ error: %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return
        pass

    def update_listener(self, old_listener, listener, service):
        self._set_listener_status(listener, service, plugin_const.PENDING_UPDATE)
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("update listener error: %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return

        self._update_service_status(service)
        pass

    def delete_listener(self, listener, service):
        self._set_listener_status(listener, service, plugin_const.PENDING_DELETE)
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("delet listener failed: %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return

    def create_pool(self, pool, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("create_pool failed %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return


    def _getSchedMethod(self, pool):
        method = 'NODE_LB_HASH'
        schedMap = {
                'ROUND_ROBIN': 'NODE_LB_WRR',
                'LEAST_CONNECTIONS':'NODE_LB_LEAST_CONN'
                }
        if pool['lb_algorithm'] in schedMap:
            method = schedMap[pool['lb_algorithm']]
        return method

    def update_pool(self, oldpool, pool, service):
        try:
            self._common_service_handler(service)
        except Exception as e :
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            LOG.error("update pool %s" % traceback.format_exc())
        pass

    def delete_pool(self, pool, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            LOG.error("error  %s" % traceback.format_exc())
        pass

    def create_member(self, member, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            LOG.error("createMember err %s" % traceback.format_exc())
        pass
    def update_member(self, old_member, member, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            LOG.error("error %s" % traceback.format_exc())
        pass

    def delete_member(self, member, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            LOG.error("deleteMember eror %s" % traceback.format_exc())
        pass

    def create_health_monitor(self, health_monitor, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("error %s " % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
        pass

    def update_health_monitor(self, old, health_monitor, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("error %s " % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
        pass

    def delete_health_monitor(self, health_monitor, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("deleteMonitor error %s ", traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
        pass

    def create_l7policy(self, l7policy, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("create l7policy error: %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return
        pass

    def update_l7policy(self, old_l7policy, l7policy, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("update l7policy error: %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return

        self._update_service_status(service)
        pass

    def delete_l7policy(self, l7policy, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("delete l7policy failed: %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return

    def create_l7rule(self, l7rule, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("create l7rule error: %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return
        pass

    def update_l7rule(self, old_l7rule, l7rule, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("update l7rule error: %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return

        self._update_service_status(service)
        pass

    def delete_l7rule(self, l7rule, service):
        try:
            self._common_service_handler(service)
        except Exception as e:
            LOG.error("delete l7rule failed: %s" % traceback.format_exc())
            self._set_status(service['loadbalancer'], plugin_const.ERROR)
            self._update_service_status(service)
            return
        self._update_service_status(service)

    def tunnel_update(self, **kargs):
        pass

    def tunnel_sync(self):
        # 从AD获取静态IP信息
        if self.conf.advertised_tunnel_wan:
            try:
                wan_info = self.ad.getWanInfo(self.conf.advertised_tunnel_wan)
            except Exception:
                wan_info = None
            if wan_info:
                vteps = set(self.conf.advertised_tunnel_ips)
                for ip in wan_info.ip:
                    vtep = self.netaddr_format(ip.ip.split('/')[0])
                    vteps.add(vtep)
                self.agent_configurations["tunneling_ips"] = list(vteps)

        # 1、通知AD的vteps
        for tunnel_ip in self.agent_configurations["tunneling_ips"]:
            self._tunnel_rpc.tunnel_sync(self._context, tunnel_ip, 'vxlan')

        # 2、更新AD的vteps
        vxlan_name = 'openstack_vxnet'
        port = cfg.CONF.vxlan_remote_port
        vteps = self.plugin_rpc.get_vteps()
        vxnet = self.ad.getVxnetinfo(vxlan_name)
        if not vxnet:
            self.ad.createVxnet(vxlan_name, port, vteps)
        else:
            old_vteps = set(vxnet.vteps)
            new_vteps = set(vteps)
            if not old_vteps.symmetric_difference(new_vteps)\
                    and vxnet.port == port:
                return
            self.ad.updateVxnet(vxlan_name, vxlan_name, port, vteps)

    def add_fdb_entries(self, fdb):
        pass

    def remove_fdb_entries(self, fdb):
        pass

    def update_fdb_entries(self, fdb):
        pass

    @log_helpers.log_method_call
    def _update_service_status(self, service):
        """Update status of objects in OpenStack """
        if not self.plugin_rpc:
            LOG.error("Cannot update status in Neutron without "
                      "RPC handler.")
            return

        self._update_elements_status(service, "l7rules", onUpdate=lambda e: {'l7policy_id': e['l7policy_id']})
        self._update_elements_status(service, "l7policies")
        self._update_elements_status(service, "members")
        self._update_elements_status(service, "healthmonitors")
        self._update_elements_status(service, "pools")
        self._update_elements_status(service, "listeners")
        self._update_element_status(service, "loadbalancer")

    def _set_status(self, element, status):
        if 'provisioning_status' in element and element['provisioning_status'] != plugin_const.PENDING_DELETE:
            LOG.debug("[----------]set status %s to id %s" % (status, element.get("id", "None")))
            element['provisioning_status'] = status
        else:
            LOG.debug("[----------]set status failed to id %s" % element.get("id", "None"))

    def _notify_update(self, element, method, onUpdate=None):
        notify = getattr(self.plugin_rpc, "update_%s_status" % method)
        args = {} if not onUpdate else onUpdate(element)
        ID = element["id"]
        status = element["provisioning_status"]
        notify(ID, status, lb_const.ONLINE, **args)

    def _notify_destroy(self, element, method, onDelete=None):
        notify = getattr(self.plugin_rpc, "%s_destroyed" % method)
        args = {} if not onDelete else onDelete(element)
        ID = element["id"]
        notify(ID, **args)

    def _update_common_status(self, element, method, onUpdate=None, onDelete=None):
        if 'provisioning_status' in element:
            status = element['provisioning_status']
            if status == plugin_const.PENDING_DELETE:
                self._notify_destroy(element, method, onDelete)
            else:
                self._notify_update(element, method, onUpdate)

    def _update_element_status(self, service, name):
        if name not in service:
            return

        element = service[name]
        self._update_common_status(element, self.method[name])

    def _update_elements_status(self, service, name, onUpdate=None, onDelete=None):
        if name not in service:
            return

        elements = service[name].values()
        for e in elements:
            self._update_common_status(e, self.method[name], onUpdate=onUpdate, onDelete=onDelete)

    def _adjust_common_status(self, element, mode):
        if 'provisioning_status' not in element:
            return
        if mode == "reset":
            self._set_status(element, plugin_const.PENDING_CREATE)
        elif mode == "refresh":
            if element['provisioning_status'] == plugin_const.ERROR:
                self._set_status(element, plugin_const.PENDING_CREATE)

    def _adjust_element_status(self, service, name, mode):
        if name not in service:
            return
        element = service[name]
        self._adjust_common_status(element, mode)

    def _adjust_elements_status(self, service, name, mode):
        if name not in service:
            return
        for element in service[name].values():
            self._adjust_common_status(element, mode)

    def _adjust_status(self, service, mode = "refresh"):
        self._adjust_element_status(service, 'loadbalancer', mode)
        self._adjust_elements_status(service, 'listeners', mode)
        self._adjust_elements_status(service, 'pools', mode)
        self._adjust_elements_status(service, 'members', mode)
        self._adjust_elements_status(service, 'healthmonitors', mode)
        self._adjust_elements_status(service, 'l7policies', mode)
        self._adjust_elements_status(service, 'l7rules', mode)

    def _refresh_status(self, service):
        self._adjust_status(service, "refresh")

    def _reset_status(self, service):
        self._adjust_status(service, "reset")

    @log_helpers.log_method_call
    def sync(self, service, mode = "refresh"):
        #调整ERROR状态配置
        if mode == "refresh":
            self._refresh_status(service)
        elif mode == "reset":
            self._reset_status(service)

        try:
            self._common_service_handler(service)
        except Exception as e:
            #self._set_status(service['loadbalancer'], plugin_const.ERROR)
            #self._update_service_status(service)
            LOG.error("sync failed: %s" % traceback.format_exc())
            raise Exception(e)

    @log_helpers.log_method_call
    def sync_check_service(self, service):
        data = ServiceBuilder(self, service).build()
        LOG.debug("[driver]start check service now: %s" % data)
        retmsg = self.ad.checkService(data)
        if retmsg["err_code"] != 0:
            LOG.debug("[driver] err_code %d, err_info %s" % (retmsg["err_code"], retmsg["err_info"]))
            if retmsg["err_code"] != 105 and retmsg["err_code"] != 106:
                return True
        else:
            return True

        try:
            self.sync(service, "reset")
            return True
        except:
            pass

        return False

    def sync_check_uuid(self, uuid):
        data = UuidBuilder(uuid).build()
        LOG.debug("[driver]start check uuid now: %s" % data)
        retmsg = self.ad.checkService(data)
        LOG.debug("[driver] err_code %d, err_info %s" % (retmsg["err_code"], retmsg["err_info"]))

    @log_helpers.log_method_call
    def _common_service_handler(self, service):
        '''
        配置下发通知接口，保留原有形式，修改内部实现
        '''
        try:
            self._assure_loadbalancer(service)
            self._assure_listeners(service)
            self._assure_pools(service)
            self._assure_l7policy(service)
            self._assure_l7rule(service)
            self._assure_members(service)
            self._assure_monitors(service)

            #状态更新可以完全复用
            self._update_service_status(service)
        except Exception as e:
            LOG.error('_common_service_handler Exception: %s'
                      % traceback.format_exc())
            raise Exception(e)

    def _assure_routes(self, subnet, netns_id, netns_uuid):
        '''
        下发相关路由
        '''
        try:
            route_set = []
            gateway = subnet['gateway_ip'] # type: str
            if gateway is not None:
                if ipv6.valid_str(gateway):
                    route_set.append({'net': '0:0:0:0:0:0:0:0/0', 'gw': gateway})
                else:
                    route_set.append({'net': '0.0.0.0/0', 'gw': gateway})
            for sub_route in subnet['host_routes']:
                (net, mask) = sub_route['destination'].split('/')
                net = self.netaddr_format(net)
                route_set.append({'net':net+'/'+mask,\
                        'gw':sub_route['nexthop']})

            LOG.debug("route set:%s" % route_set)
            for route in route_set:
                self.ad.createRoute(route['net'], netns_id, self.netaddr_format(route['gw']), netns_uuid)

        except Exception as e:
            LOG.error('assure route faild : %s'% traceback.format_exc())
            raise Exception(e)
    
    def netaddr_format(self, addr):
        if not addr:
            return addr

        if ipv6.valid_str(addr):
            return ipv6.int_to_str(ipv6.str_to_int(addr), ipv6.ipv6_full)
        else:
            return addr

    def _createLoadbalancer(self, lb, service):
        try:
            subnetsinfo = self._get_subnets_to_assure(service)
            bridge = self._assure_network(subnetsinfo[0]['network'], service)
            netns_info = self._assure_netns(lb)
            if bridge[0] == 'flat':
                bridge[1] = self._get_interface(subnetsinfo[0]['network'])
            macvlan_name = self._get_loadbalancer_name(lb)
            mac_addr = lb['vip_port']['mac_address']
            macvlan_device = self._assure_macvlan(netns_info['netnsId'],
                                                  bridge[1], macvlan_name, mac_addr, lb['id'])

            ipg_name = self._get_loadbalancer_name(lb)
            vip = self.netaddr_format(service['loadbalancer']['vip_address'])
            if ipv6.valid_str(vip):
                gw_default = '::'
            else:
                gw_default = "0.0.0.0"
            gw = self.netaddr_format(subnetsinfo[0]['subnet'].get('gateway_ip', gw_default))
            if gw is None:
                gw = gw_default
            self.ad.createIPGroup(ipg_name, [vip],\
                    {'mask':int(subnetsinfo[0]['subnet']['cidr'].split('/')[1]), \
                    'gatewayIP':gw, 'wan':ipg_name,
                    'ifname':macvlan_device, 'info_id':netns_info['appgId']}, lb['id'])

            #最后下发相关路由
            self._assure_routes(subnetsinfo[0]['subnet'], netns_info['netnsId'], lb['id'])

        except Exception as e:
            LOG.error('_createLoadbalancer Error %s' % traceback.format_exc())
            raise Exception(e)

    def _delete_network_vlan(self, network):
        vlan_name = self._get_vlan_name(network['id'])
        LOG.debug("vlan name is %s"%vlan_name)
        self.ad.deleteVlan(vlan_name)

    def _delete_network_vxlan(self, network):
        vxlan_name = self._get_vxlan_name(network['id'])
        LOG.debug("vxlan name is %s" % vxlan_name)
        self.ad.deleteVxlan(vxlan_name)
        vteps = self.get_tunnel_ips()
        for ip in vteps:
            self.notify_vtep_removed(network, ip)

    def _clean_network(self, network, service):
        if not network:
            LOG.error("no network_id in lb")
            return

        LOG.debug("network info %s" % network)
        if network['provider:network_type'] == 'vlan':
            self._delete_network_vlan(network)
        elif network['provider:network_type'] == 'vxlan':
            self._delete_network_vxlan(network)
        elif network['provider:network_type'] == 'flat':
            pass
        else:
            return

    def _deleteLoadbalancer(self, lb, service):
        try:
            lbname = self._get_loadbalancer_name(lb)
            netns_id = self.ad.getNetnsId(lbname)
            if netns_id:
                #1、删除路由
                self.ad.clearRouteInNetns(netns_id)
                #2、删除ip组
                self.ad.deleteIPGroup(lbname)
                #3、删除wan口
                self.ad.deleteWan(lbname)
                #4、删除macvlan和netns
                self.ad.deleteMacvlan(lbname)
                self.ad.deleteNetns(lbname)
                #5、TODO:需要反查是否vlan对应的network已无lb引用，如果是则删除vlan
                self._clean_network(service['networks'][lb['network_id']],
                                    service)
        except Exception as e:
            LOG.error('_deleteLoadbalancer Error %s' % traceback.format_exc())
            # 暂时不再raise错误, 直接清除数据库
            raise Exception(e)

    def _assure_loadbalancer(self, service):
        '''
        lb下发接口
        '''
        if 'loadbalancer' not in service:
            return
        lb = service['loadbalancer']
        name = self._get_loadbalancer_name(lb)
        status = lb['provisioning_status']
        #TODO: 如果单台AD的话，delete需要处理一下了
        try:
            if status == plugin_const.PENDING_DELETE:
                self._deleteLoadbalancer(lb, service)
            elif status == plugin_const.PENDING_CREATE:
                self._createLoadbalancer(lb, service)
            elif status != plugin_const.PENDING_UPDATE:
                return
        except Exception as e:
            self._set_status(lb, plugin_const.ERROR)
            raise Exception(e)

        self._set_status(lb, plugin_const.ACTIVE)

    def _createListener(self, loadbalancer, listener):
        self._createDefaultPool(_DEFAULT_POOL)
        ipg_name = self._get_loadbalancer_name(loadbalancer)
        name = self._get_listener_name(listener)
        if listener['protocol'] == 'HTTP':
            srvtype = 'SRV_HTTP'
            mode = 'VS_MODE_L7'
        elif listener['protocol'] == 'HTTPS':
            srvtype = 'SRV_HTTPS'
            mode = 'VS_MODE_L7'
            if not listener.get('default_tls_container_id', None):
                raise Exception("Use HTTPS, But no default_tls_container_id specified")
        else:
            srvtype = 'SRV_TCP'
            mode = 'VS_MODE_L4'
        #srvtype = 'SRV_HTTP' if listener['protocol'] == 'HTTP' else 'SRV_TCP'
        enable = 'true' if listener['admin_state_up'] else 'false'
        #mode = 'VS_MODE_L7' if listener['protocol'] == 'HTTP' else 'VS_MODE_L4'
        #获取一下netns_id给虚拟服务创建使用
        netns_id = self.ad.getNetnsId(ipg_name)
        self.ad.createServ(name, srvtype, [int(listener['protocol_port'])], loadbalancer['id'])
        self.ad.createVS(name, {'poolName': _DEFAULT_POOL, "ipgName": ipg_name,
                                'serviceName': name, 'enable': enable, 'mode':mode,
                                'netns_id': netns_id, 'limit': listener.get('connection_limit', -1),
                                'x_forward_enable': self.conf.x_forwarded_for_enable,
                                'ssl_profile': listener.get('default_tls_container_id', None)},
                         loadbalancer['id'])

    def _updateListener(self, loadbalancer, listener):
        #vsname = 'openstack-'+listener['id']
        vsname = self._get_listener_name(listener)
        enable = 'true' if listener['admin_state_up'] else 'false'
        limit = listener.get('connection_limit', -1)
        self.ad.updateVS(vsname, {'enable': enable, 'limit': limit, 'ssl_profile': listener.get('default_tls_container_id', None)})

    def _deleteListener(self, loadbalancer, listener):
        vsname = self._get_listener_name(listener)
        self.ad.deleteVS(vsname)
        self.ad.deleteServ(vsname)

    def _assure_listeners(self, service):
        if 'listeners' not in service:
            return

        listeners = service['listeners'].values()
        loadbalancer = service['loadbalancer']
        for listener in listeners:
            try:
                if listener['provisioning_status'] == plugin_const.PENDING_CREATE:
                    self._createListener(loadbalancer, listener)
                elif listener['provisioning_status'] == plugin_const.PENDING_UPDATE:
                    self._updateListener(loadbalancer, listener)
                elif listener['provisioning_status'] == plugin_const.PENDING_DELETE:
                    self._deleteListener(loadbalancer, listener)
                else:
                    continue
            except Exception as e:
                self._set_status(listener, plugin_const.ERROR)
                raise Exception(e)

            self._set_status(listener, plugin_const.ACTIVE)

    def _createPool(self, service, pool):
        loadbalancer = service['loadbalancer']
        lbname = self._get_loadbalancer_name(loadbalancer)
        poolname = self._get_pool_name(pool)
        method = self._getSchedMethod(pool)
        relat = False if pool['protocol'] != 'HTTP' and \
                pool['session_persistence'] and \
                'type' in pool['session_persistence'] and \
                pool['session_persistence']['type'] != 'SOURCE_IP' else True
        netns_id = self.ad.getNetnsId(lbname)
        netns_uuid = loadbalancer['id']
        if pool['session_persistence'] and relat:
            if pool['session_persistence']['type'] == 'SOURCE_IP':
                self.ad.createPersist(poolname, {'type': 'PERSIST_SOURCE_IP'}, netns_uuid)
            elif pool['session_persistence']['type'] == 'HTTP_COOKIE':
                self.ad.createPersist(poolname, \
                        {'type':'PERSIST_COOKIE', \
                        'cookieType':'PERSIST_COOKIE_INSERT',\
                        'cookieName':'openstack_cookie_insert'}, netns_uuid)
            else:
                self.ad.createPersist(poolname, \
                        {'type':'PERSIST_COOKIE', \
                        'cookieType':'PERSIST_COOKIE_STUDY',\
                        'cookieName':pool['session_persistence']['cookie_name']}, netns_uuid)
            self.ad.createPool(poolname, {'lbMethod':method, 'persist1Name':poolname,\
                    'netns_id':netns_id}, netns_uuid)
        else:
            self.ad.createPool(poolname, {'lbMethod':method, 'netns_id':netns_id}, netns_uuid)
        for listener_id in pool['listeners']:
            self.ad.updateVS(self._get_listener_name(service['listeners'][listener_id]), {'poolName' : poolname})

    def _updatePool(self, service, pool):
        loadbalancer = service['loadbalancer']
        pool_name = self._get_pool_name(pool)
        method = self._getSchedMethod(pool)
        netns_uuid = loadbalancer['id']
        relat = False if pool['protocol'] != 'HTTP' and \
                pool['session_persistence'] and \
                'type' in pool['session_persistence'] and \
                pool['session_persistence']['type'] != 'SOURCE_IP' else True
        if pool['session_persistence'] and relat:
            try:
                self.ad.updatePool(pool_name,  {'persist1Name':'none'})
                self.ad.deletePersist(pool_name)
            except Exception as e:
                LOG.debug(e)
            if pool['session_persistence']['type'] == 'SOURCE_IP':
                self.ad.createPersist(pool_name, {'type': 'PERSIST_SOURCE_IP'}, netns_uuid)
                self.ad.updatePool(pool_name, {'persist1Name': pool_name, 'lbMethod':method})
            elif pool['session_persistence']['type'] == 'HTTP_COOKIE':
                self.ad.createPersist(pool_name, \
                        {'type':'PERSIST_COOKIE', \
                        'cookieType':'PERSIST_COOKIE_INSERT',\
                        'cookieName':'openstack_cookie_insert'}, netns_uuid)
                self.ad.updatePool(pool_name, {'persist1Name':pool_name, 'lbMethod':method})
            else:
                self.ad.createPersist(pool_name, \
                        {'type':'PERSIST_COOKIE', \
                        'cookieType':'PERSIST_COOKIE_STUDY',\
                        'cookieName':pool['session_persistence']['cookie_name']}, netns_uuid)
                self.ad.updatePool(pool_name, \
                        {'persist1Name':pool_name, 'lbMethod':method})
        else:
            self.ad.updatePool(pool_name, {'persist1Name':'none', 'lbMethod':method})

    def _deletePool(self, service, pool):
        pool_name = self._get_pool_name(pool)
        if pool['session_persistence'] :
            self.ad.updatePool(pool_name,  {'persist1Name':'none'})
            self.ad.deletePersist(pool_name)
        if pool['healthmonitor_id']:
            monitor_name = self._get_monitor_name(service['healthmonitors'][pool['healthmonitor_id']])
            self.ad.delPoolMonitor(pool_name, monitor_name)
            self.ad.deleteMonitor(monitor_name)
        for listener_id in pool['listeners']:
            self.ad.updateVS(self._get_listener_name(service['listeners'][listener_id])\
                    , {'poolName':_DEFAULT_POOL})
        self.ad.deletePool(pool_name)

    def _assure_pools(self, service):
        if 'pools' not in service:
            return

        pools = service['pools'].values()

        for pool in pools:
            try:
                if pool['provisioning_status'] == plugin_const.PENDING_CREATE:
                    self._createPool(service, pool)
                elif pool['provisioning_status'] == plugin_const.PENDING_UPDATE:
                    self._updatePool(service, pool)
                elif pool['provisioning_status'] == plugin_const.PENDING_DELETE:
                    self._deletePool(service, pool)
                else:
                    continue
            except Exception as e:
                self._set_status(pool, plugin_const.ERROR)
                LOG.error("%s" % traceback.format_exc())
                raise Exception(e)

            self._set_status(pool, plugin_const.ACTIVE)

    def _translatePolicy(self, policy, service):
        kargs = {}
        kargs['service'] = self._get_listener_name(service['listeners'][policy['listener_id']])
        kargs['enable'] = policy.get('admin_state_up', False)
        if policy['action'] == "REJECT":
            kargs['rule_action'] = "RULE_ACTION_DROP"
        elif policy['action'] == "REDIRECT_TO_POOL":
            kargs['rule_action'] = "RULE_ACTION_SCHED"
            kargs['pool_name'] = self._get_pool_name(service['pools'][policy['redirect_pool_id']])
        elif policy['action'] == 'REDIRECT_TO_URL':
            kargs['rule_action'] = "RULE_ACTION_REDIR_URL"
            kargs['redir_url'] = policy['redirect_url']
        else:
            assert 0

        rules = []
        for r in policy.get('rules', []):
            r = service['l7rules'][r]
            rule = {}

            type = r['type']
            compare_type = r['compare_type']
            header = r.get('key')
            value = r['value']

            if r['type'] == "PATH":
                type = "URI"
                compare_type = r['compare_type']
            elif r['type'] == 'FILE_TYPE':
                type = "URI"
                compare_type = "ENDS_WITH"
            elif r['type'] == 'HOST_NAME':
                type = "HEADER"
                header = "Host"
            elif r['type'] == 'COOKIE':
                type = "HEADER"
                header = "Cookie"
                K = re.escape(r['key'])
                V = re.escape((r['value']))
                if compare_type == "STARTS_WITH":
                    value = "(.*;|^)\s*%s\s*=\s*%s.*" % (K, V)
                elif compare_type == "ENDS_WITH":
                    value = "(.*;|^)\s*%s\s*=\s*[^;]*%s(;|$)" % (K, V)
                elif compare_type == "EQUAL_TO":
                    value = "(.*;|^)\s*%s\s*=\s*%s(;|$)" % (K, V)
                elif compare_type == "CONTAINS":
                    value = "(.*;|^)\s*%s\s*=[^;]*%s" % (K, V)
                else:
                    value = "(.*;|^)\s*%s\s*=\s*%s" % (r['key'], r['value'])
                compare_type = 'REGEX'

            V = re.escape(r['value'])
            if compare_type == "STARTS_WITH":
                compare_type = "STR_MATCH_TYPE_REGEX"
                value = "^%s.*" % V
            elif compare_type == "ENDS_WITH":
                compare_type = "STR_MATCH_TYPE_REGEX"
                value = ".*%s$" % V
            elif compare_type == "REGEX":
                compare_type = "STR_MATCH_TYPE_REGEX"
            elif compare_type == "EQUAL_TO":
                compare_type = "STR_MATCH_TYPE_EQUAL"
            elif compare_type == "CONTAINS":
                compare_type = "STR_MATCH_TYPE_CONTAIN"
            else:
                compare_type = "STR_MATCH_TYPE_NONE"

            rule['type'] = type
            if header is not None:
                rule['key'] = header
            rule['compare_type'] = compare_type
            rule['value'] = value
            rule['invert'] = r['invert']

            rules.append(rule)

        kargs['rules'] = rules
        return kargs

    def _get_policy_name(self, policy):
        if policy['name'] != "":
            return "%s-%s" % (policy['name'], policy['id'])
        return "openstack-" + policy['id']

    def _createPolicy(self, policy, service):
        lb = service['loadbalancer']
        policy_name = self._get_policy_name(policy)

        kargs = self._translatePolicy(policy, service)
        kargs['netns_uuid'] = lb['id']

        self.ad.createSchedRule(policy_name, kargs)

    def _updatePolicy(self, policy, service):
        policy_name = self._get_policy_name(policy)
        kargs = self._translatePolicy(policy, service)
        self.ad.updateSchedRule(policy_name, kargs)

    def _deletePolicy(self, policy, service):
        vs_name = self._get_listener_name(service['listeners'][policy['listener_id']])
        policy_name = self._get_policy_name(policy)
        self.ad.deleteVsSched(vs_name, policy_name)
        self.ad.deleteSchedRule(policy_name)

    def _assure_l7policy(self, service):
        if 'l7policies' not in service:
            return

        l7policies = service['l7policies'].values()

        listeners = set()

        for policy in l7policies:
            rules = []
            for rule_id in policy['rules']:
                rule = service['l7rules'][rule_id]
                LOG.debug("[RULE]: %s" % rule)
                if rule['provisioning_status'] != plugin_const.PENDING_DELETE:
                    rules.append(rule_id)
                if rule['provisioning_status'] != plugin_const.ACTIVE and \
                   policy['provisioning_status'] == plugin_const.ACTIVE:
                    policy['provisioning_status'] = plugin_const.PENDING_UPDATE
            policy['rules'] = rules

            try:
                if policy['provisioning_status'] == plugin_const.PENDING_CREATE:
                    listeners.add(policy['listener_id'])
                    self._createPolicy(policy, service)
                elif policy['provisioning_status'] == plugin_const.PENDING_UPDATE:
                    self._updatePolicy(policy, service)
                elif policy['provisioning_status'] == plugin_const.PENDING_DELETE:
                    self._deletePolicy(policy, service)
                else:
                    continue
            except Exception as e:
                self._set_status(policy, plugin_const.ERROR)
                LOG.error("%s" % traceback.format_exc())
                raise Exception(e)

            self._set_status(policy, plugin_const.ACTIVE)

        for ls_id in listeners:
            self._updateVsProfile(ls_id, service)

    def _updateVsProfile(self, ls_id, service):
        listener = service['listeners'][ls_id]
        try:
            listener['l7policies'].sort(key=lambda p: service['l7policies'][p]['position'])
            preProfile = [self._get_policy_name(service['l7policies'][p]) for p in listener['l7policies']]
            self.ad.updateVS(self._get_listener_name(listener), {'preProfile': preProfile})
        except Exception as e:
            self._set_status(listener, plugin_const.ERROR)
            LOG.error("%s" % traceback.format_exc())
            raise Exception(e)

    def _assure_l7rule(self, service):
        if 'l7rules' not in service:
            return

        l7rules = service['l7rules'].values()

        for rule in l7rules:
            self._set_status(rule, plugin_const.ACTIVE)

    def _createMember(self, service, member):
        pool_name = self._get_pool_name(service['pools'][member['pool_id']])
        self.ad.createMember(pool_name, self.netaddr_format(member['address']), member['protocol_port'], \
            {'ratio':member['weight'], 'enable':member['admin_state_up']})
        pass

    def _updateMember(self, service, member):
        pool_name = self._get_pool_name(service['pools'][member['pool_id']])
        self.ad.updateMember(pool_name, self.netaddr_format(member['address']), member['protocol_port'], \
            {'ratio':member['weight'], 'enable':member['admin_state_up']})
        pass

    def _deleteMember(self, service, member):
        pool_name = self._get_pool_name(service['pools'][member['pool_id']])
        self.ad.deleteMember(pool_name, self.netaddr_format(member['address']), member['protocol_port'])
        pass

    def _createMonitor(self, monitor, service, netns_uuid):
        name = self._get_monitor_name(monitor)
        mtype= "MONITOR_ICMPV4" if monitor['type'] not in _MONITOR_TYPE_MAP else _MONITOR_TYPE_MAP[monitor['type']]
        tryout = monitor['max_retries'] if int(monitor['max_retries'] ) < _MONITOR_RETRIES else _MONITOR_RETRIES
        timeout = monitor['timeout'] if int(monitor['timeout']) < _MONTTOR_MAX_TIMEOUT else _MONTTOR_MAX_TIMEOUT
        interval = monitor['delay'] if int(monitor['delay']) < _MONITOR_MAX_DELAY else _MONITOR_MAX_DELAY
        args={
                'interval':interval,
                'timeout':timeout,
                'tryout':tryout
            }

        if monitor.get('expected_codes', None) != None:
            args['respCode'] = monitor['expected_codes']
        if monitor.get('url_path', None) != None:
            args['urlMsg'] = monitor['url_path']
        if monitor.get('http_method', None) != None:
            args['method'] = monitor['http_method']

        if mtype == "MONITOR_CONNECT_TCP":
            self.ad.createMonitor(name, mtype, args, netns_uuid)
        elif mtype == 'MONITOR_ICMPV4':
            self.ad.createMonitor(name,mtype, args, netns_uuid)
        elif mtype == 'MONITOR_CONNECT_HTTP':
            args['respCode']=monitor['expected_codes']
            args['urlMsg']=monitor['url_path']
            self.ad.createMonitor(name, mtype, args, netns_uuid)
        elif mtype == 'MONITOR_CONNECT_HTTPS':
            args['resp_code']=monitor['expected_codes']
            args['url_msg']=monitor['url_path']
            self.ad.createMonitor(name, mtype, args, netns_uuid)

        #for monitor in service['healthmonitors']:
        self.ad.addPoolMonitor(self._get_pool_name(service['pools'][monitor['pool_id']]), name)
        pass

    def _updateMonitor(self, monitor):
        name = self._get_monitor_name(monitor)
        interval = monitor['delay'] if int(monitor['delay']) < _MONITOR_MAX_DELAY else _MONITOR_MAX_DELAY
        tryout = monitor['max_retries'] if int(monitor['max_retries'] ) < _MONITOR_RETRIES else _MONITOR_RETRIES
        timeout = monitor['timeout'] if int(monitor['timeout']) < _MONTTOR_MAX_TIMEOUT else _MONTTOR_MAX_TIMEOUT
        mtype= "PING" if monitor['type'] not in _MONITOR_TYPE_MAP else _MONITOR_TYPE_MAP[monitor['type']]
        args={
                'type':mtype,
                'interval':interval,
                'timeout':timeout,
                'tryout':tryout
            }

        if monitor.get('expected_codes', None) != None:
            args['respCode'] = monitor['expected_codes']
        if monitor.get('url_path', None) != None:
            args['urlMsg'] = monitor['url_path']
        if monitor.get('http_method', None) != None:
            args['method'] = monitor['http_method']

        if mtype == "MONITOR_CONNECT_TCP":
            self.ad.updateMonitor(name, mtype, args)
        elif mtype == 'MONITOR_ICMPV4':
            self.ad.updateMonitor(name, mtype, args)
        elif mtype == 'MONITOR_CONNECT_HTTP':
            args['respCode']=monitor['expected_codes']
            args['urlMsg']=monitor['url_path']
            self.ad.updateMonitor(name, mtype, args)
        elif mtype == 'MONITOR_CONNECT_HTTPS':
            args['respCode']=monitor['expected_codes']
            args['urlMsg']=monitor['url_path']
            self.ad.updateMonitor(name, mtype, args)
        pass

    def _deleteMonitor(self, monitor, service):
        name = self._get_monitor_name(monitor)
        #for monitor in service'healthmonitors']:
        self.ad.delPoolMonitor(self._get_pool_name(service['pools'][monitor['pool_id']]), name)
        self.ad.deleteMonitor(name)
        pass

    def _assure_members(self, service):
        if not (('pools' in service) and ('members' in service)):
            return
        members = service['members'].values()

        for member in members:
            try:
                if member['provisioning_status'] == plugin_const.PENDING_CREATE:
                    try:
                        self._createMember(service, member)
                    except:
                        LOG.error("member:%s create error" % member["id"])
                elif member['provisioning_status'] == plugin_const.PENDING_UPDATE or member['provisioning_status'] == plugin_const.INACTIVE:
                    self._updateMember(service, member)
                elif member['provisioning_status'] == plugin_const.PENDING_DELETE:
                    try:
                        self._deleteMember(service, member)
                    except:
                        LOG.error("member:%s delete error " % member["id"])
                else:
                    continue
            except Exception as e:
                LOG.error("assure member error: %s" % traceback.format_exc())
                self._set_status(member, plugin_const.ERROR)
                raise e

            self._set_status(member, plugin_const.ACTIVE)

        # 双臂模式，根据member子网决定是否申请lan口
        try:
            self._update_snat_network(service)
        except Exception as e:
            LOG.error("update snat network failed: %s" % traceback.format_exc())
            raise e

    def _assure_monitors(self, service):
        if not (('pools' in service) and ('healthmonitors' in service)):
            return
        monitors = service['healthmonitors'].values()
        loadbalancer = service['loadbalancer']

        for monitor in monitors:
            try:
                if monitor['provisioning_status'] == plugin_const.PENDING_CREATE:
                    self._createMonitor(monitor, service, loadbalancer['id'])
                elif monitor['provisioning_status'] == plugin_const.PENDING_UPDATE:
                    self._updateMonitor(monitor)
                elif monitor['provisioning_status'] == plugin_const.PENDING_DELETE:
                    self._deleteMonitor(monitor, service)
                else:
                    continue
            except Exception as e:
                self._set_status(monitor, plugin_const.ERROR)
                raise Exception(e)

            self._set_status(monitor, plugin_const.ACTIVE)

    def get_snat_network(self, port, network):
        return self.plugin_rpc.get_network_segment(
                agent_config=self.agent_configurations,
                host=port[portbindings.HOST_ID],
                port_id=port['id'],
                network=network)

    def _create_snat_port(self, port, service, netns_uuid):
        loadbalancer = service['loadbalancer']
        network = service['networks'][port['network_id']]
        subnet = service['subnets'][port['fixed_ips'][0]['subnet_id']]
        """
        当lan口和wan口不在同一个网络时，需要重新获取层次化绑定信息
        因为直接从service里面取到的是member的层次化绑定信息
        """
        if port['network_id'] != service['loadbalancer']['network_id']:
            network = self.get_snat_network(port, network)
        bridge = self._assure_network(network, service)
        if bridge[0] == 'flat':
            bridge[1] = self._get_interface(network)
        netns_name = self._get_loadbalancer_name(loadbalancer)
        netns_info = self.ad.getNetnsInfo(netns_name)
        macvlan_name = port['name']
        mac_addr = port['mac_address']
        macvlan_device = self._assure_macvlan(netns_info['netnsId'],
                                              bridge[1],
                                              macvlan_name,
                                              mac_addr,
                                              netns_uuid)
        self.ad.createLan(port['name'], macvlan_device,
                          [self.netaddr_format(port['fixed_ips'][0]['ip_address'])],
                          {'mask': int(subnet['cidr'].split('/')[1]),
                           'info_id': netns_info['appgId']}, netns_uuid)

    def _destroy_snat_port(self, port, service):
        network = service['networks'][port['network_id']]
        """
        当lan口和wan口不在同一个网络时，需要重新获取层次化绑定信息
        因为直接从service里面取到的是member的层次化绑定信息
        """
        if port['network_id'] != service['loadbalancer']['network_id']:
            network = self.plugin_rpc.get_network_segment(
                agent_config=self.agent_configurations,
                host=port[portbindings.HOST_ID],
                port_id=port['id'],
                network=network)
        self.ad.deleteLan(port['name'])
        self.ad.deleteMacvlan(port['name'])
        #5、TODO:需要反查是否vlan对应的network已无lb引用，如果是则删除vlan
        self._clean_network(network, service)

    def get_snat_port_name(self, loadbalancer_id, sunet_id):
        # 如果说这里存在冲突的可能性，那么uuid本身就存在冲突可能性吧
        return '%s-%s-%s' \
               % ('op-snat', loadbalancer_id, sunet_id[:8])

    def get_pool_provisioning_status(self, service, member):
        return service['pools'][member['pool_id']]['provisioning_status']

    def get_snat_ports(self, service):
        snat_ports = []
        members = service['members'].values()
        vip_subnet = service['loadbalancer']['vip_subnet_id']
        loadbalancer_id = service['loadbalancer']['id']
        subnets = set()
        for member in members:
            member_subnet = member['subnet_id']
            # 当pool正在删除时，将所有member按待删除计算
            if self.get_pool_provisioning_status(service, member) != plugin_const.PENDING_DELETE \
                    and member['provisioning_status'] != plugin_const.PENDING_DELETE:
                subnets.add(member_subnet)
        if vip_subnet in subnets:
            subnets.remove(vip_subnet)
        for subnet in subnets:
            port_name = self.get_snat_port_name(loadbalancer_id, subnet)
            ports = self.plugin_rpc.get_port_by_name(port_name=port_name)
            if ports:
                snat_ports.append(ports[0])
        return snat_ports

    def _update_snat_network(self, service):
        if not cfg.CONF.snat_port_enable:
            return
        old_snat_subnets = set()
        new_nsat_subnets = set()
        members = service['members'].values()
        vip_subnet = service['loadbalancer']['vip_subnet_id']
        loadbalancer_id = service['loadbalancer']['id']
        vip_port = service['loadbalancer']['vip_port']

        old_snat_subnets.add(vip_subnet)
        new_nsat_subnets.add(vip_subnet)
        for member in members:
            member_subnet = member['subnet_id']
            # 当pool正在删除时，将所有member按待删除计算
            if self.get_pool_provisioning_status(service, member) \
                    == plugin_const.PENDING_DELETE:
                old_snat_subnets.add(member_subnet)
            elif member['provisioning_status'] == plugin_const.PENDING_CREATE:
                new_nsat_subnets.add(member_subnet)
            elif member['provisioning_status'] == plugin_const.PENDING_DELETE:
                old_snat_subnets.add(member_subnet)
            else:
                new_nsat_subnets.add(member_subnet)
                old_snat_subnets.add(member_subnet)
        LOG.debug('update_snat_network, old_snat_subnets:%s, '
                  'new_nsat_subnets:%s, service:%s'
                  % (old_snat_subnets, new_nsat_subnets, service))
        add_snat_subnets = new_nsat_subnets.difference((vip_subnet, ))
        del_snat_subnets = old_snat_subnets.difference(new_nsat_subnets)

        for snat_subnet in add_snat_subnets:
            LOG.debug('try add snat port %s' % snat_subnet)
            try:
                port_name = self.get_snat_port_name(loadbalancer_id, snat_subnet)
                ports = self.plugin_rpc.get_port_by_name(port_name=port_name)
                port = None
                if not ports:
                    port = self.plugin_rpc.create_port_on_subnet(
                        subnet_id=snat_subnet,
                        port_attr={
                            'name': port_name,
                            'device_id': vip_port['device_id'],
                            portbindings.HOST_ID: vip_port[portbindings.HOST_ID],
                            'device_owner': vip_port['device_owner'],
                        },
                        fixed_address_count=1,
                    )
                else:
                    port = ports[0]
                if port:
                    self._create_snat_port(port, service, loadbalancer_id)
            except Exception as e:
                LOG.error('add snat port failed, %s' % traceback.format_exc())
                raise Exception(e)

        for snat_subnet in del_snat_subnets:
            LOG.debug('try del snat port %s' % snat_subnet)
            try:
                port_name = self.get_snat_port_name(loadbalancer_id, snat_subnet)
                ports = self.plugin_rpc.get_port_by_name(port_name=port_name)
                if not ports:
                    continue
                try:
                    self._destroy_snat_port(ports[0], service)
                except Exception as e:
                    LOG.error('_destroy_snat_port Exception: %s'
                              % traceback.format_exc())
                    raise e
                finally:
                    # _destroy_snat_port需要获取层次化绑定信息，因此port后释放
                    self.plugin_rpc.delete_port_by_name(port_name=port_name)
            except Exception:
                LOG.error('del snat port failed, %s' % traceback.format_exc())

    def remove_orphans(self, loadbalancer):
        pass

    def _resetProvisioningStatus(self, service):
        if ('loadbalancer' not in service)\
                or (service['loadbalancer']['provisioning_status']\
                    == plugin_const.PENDING_DELETE):
            LOG.error('service status error return')
            return

        lb = service['loadbalancer']
        lb['provisioning_status'] = plugin_const.PENDING_CREATE
        #self.plugin_rpc.update_loadbalancer_status(
        #        loadbalancer['id'],
        #        plugin_const.PENDING_CREATE)

        if 'members' in service:
            for member in service['members'].values():
                if member['provisioning_status'] == plugin_const.ACTIVE or \
                        member['provisioning_status'] == plugin_const.PENDING_UPDATE or \
                        member['provisioning_status'] == plugin_const.ERROR:
                    member['provisioning_status'] = plugin_const.PENDING_CREATE
        if 'healthmonitors' in service:
            for monitor in service['healthmonitors'].values():
                if monitor['provisioning_status'] == plugin_const.ACTIVE or \
                        monitor['provisioning_status'] == plugin_const.PENDING_UPDATE or \
                        monitor['provisioning_status'] == plugin_const.ERROR:
                    monitor['provisioning_status'] = plugin_const.PENDING_CREATE
        if 'pools' in service:
            for pool in service['pools'].values():
                if pool['provisioning_status'] == plugin_const.ACTIVE or \
                        pool['provisioning_status'] == plugin_const.PENDING_UPDATE or \
                        pool['provisioning_status'] == plugin_const.ERROR:
                    pool['provisioning_status'] = plugin_const.PENDING_CREATE
        if 'listeners' in service:
            for listener in service['listeners'].values():
                if listener['provisioning_status'] == plugin_const.ACTIVE or \
                        listener['provisioning_status'] == plugin_const.PENDING_UPDATE or \
                        listener['provisioning_status'] == plugin_const.ERROR:
                    listener['provisioning_status'] = plugin_const.PENDING_CREATE
        LOG.error('reset status, service is %s'% service)

    def exists(self, service):
        '''
        if self.reset == True:
            self.reset = False
            self._resetProvisioningStatus(service)
            return False
        '''
        lb_name = self._get_loadbalancer_name(service['loadbalancer'])
        #TODO:判定虚拟服务配置是否正常，暂时只看netns
        netns_id = self.ad.getNetnsId(lb_name)
        if netns_id:
            return True

        LOG.warn('lb %s does not exist!'%lb_name)
        self._resetProvisioningStatus(service)
        return False
