#coding=utf-8

import datetime

import hashlib
import logging as std_logging
import urllib2
import uuid

from eventlet import greenthread
from time import time

from oslo_log import helpers as log_helpers


from oslo_utils import importutils

try:
    from neutron import context as ncontext
except:
    from neutron_lib import context as ncontext

import oslo_messaging
from oslo_config import cfg
from oslo_log import log as logging
from neutron.common import topics
try:
    import neutron_lib.exceptions as  q_exception
except:
    from neutron.common import exceptions as q_exception
#from neutron.common.exceptions import InvalidConfigurationOption
#from neutron.common.exceptions import NeutronException
try:
    from neutron_lib import constants as plugin_const
except:
    from neutron.plugins.common import constants as plugin_const
from neutron.plugins.ml2.drivers.l2pop import rpc as l2pop_rpc
from neutron_lbaas.services.loadbalancer import constants as lb_const
from neutron.agent import rpc as agent_rpc
#from sangfor_openstack_agent.lbaasv2.drivers.ad.lbaas_driver import \
#    LBaaSBaseDriver
from sangfor_openstack_agent.lbaasv2.drivers.ad import constants_v2
from sangfor_openstack_agent.lbaasv2.drivers.ad import plugin_rpc

import traceback

try:
    version = 'k'
    from neutron.openstack.common import loopingcall
    from neutron.openstack.common import periodic_task
except Exception:
    version = 'm'
    from oslo_service import loopingcall
    from oslo_service import periodic_task

LOG = logging.getLogger(__name__)

OPTS = [
    cfg.StrOpt(
        'sangfor_ad_lbaas_device_driver',
        default=('sangfor_openstack_agent.lbaasv2.drivers.ad.ad_driver_physical_ad.ADDriver'),
        help=('The driver used to provision AD')
    ),
    cfg.BoolOpt(
        'sangfor_snat_mode',
        default=True,
        help=('use SNATs, not direct routed mode')
    ),

    cfg.IntOpt(
        'service_resync_interval',
        default=30,
        help=('Number of seconds between service refresh checks')
    ),
    cfg.StrOpt(
        'environment_prefix',
        default='sangfor',
        help=('The object name prefix for this environment')
    ),
    cfg.IntOpt(
        'environment_group_number',
        default=1,
        help=('Agent group number for the environment')
    ),
    cfg.BoolOpt(
        'config_check_enable',
        default=True,
        help=('Enable Config Check')
    ),
    cfg.IntOpt(
        'config_check_period',
        default=300,
        help=('Config Check Period Time')
    ),
    cfg.BoolOpt(
        'l2_population',
        default=False,
        help=('Use L2 Populate service for fdb entries on the OpenAD')
    ),
    cfg.IntOpt(
        'service_config_interval',
        default=120,
        help=('Number of seconds between service checks after config update')
    ),
    cfg.IntOpt(
        'sync_try_times',
        default=3,
        help=('times of try sync when failed')
    ),
]

class LogicalServiceCache(object):
    class Service(object):

        def __init__(self, port_id, lb_id, tenant_id, agent_host):
            self.port_id = port_id
            self.loadbalancer_id = lb_id
            self.tenant_id = tenant_id
            self.agent_host = agent_host
            self.error = 0

        def __eq__(self, other):
            return self.__dict__ == other.__dict__

        def __hash__(self):
            return hash(
                (self.port_id,
                 self.loadbalancer_id,
                 self.tenant_id,
                 self.agent_host,
                 self.error)
            )
    def __init__(self):
        """Initialize Service cache object."""
        LOG.debug("Initializing LogicalServiceCache")
        self.services = {}

    @property
    def size(self):
        """Return the number of services cached."""
        return len(self.services)

    def put(self, service, agent_host):
        """Add a service to the cache."""
        if 'port_id' in service['loadbalancer']:
            port_id = service['loadbalancer']['port_id']
        else:
            port_id = None
        loadbalancer_id = service['loadbalancer']['id']
        tenant_id = service['loadbalancer']['tenant_id']
        if loadbalancer_id not in self.services:
            s = self.Service(port_id, loadbalancer_id, tenant_id, agent_host)
            self.services[loadbalancer_id] = s
        else:
            s = self.services[loadbalancer_id]
            s.tenant_id = tenant_id
            s.port_id = port_id
            s.agent_host = agent_host
            if service['loadbalancer']['provisioning_status'] == plugin_const.ERROR:
                s.error += 1

    def remove(self, service):
        """Remove a service from the cache."""
        if not isinstance(service, self.Service):
            loadbalancer_id = service['loadbalancer']['id']
        else:
            loadbalancer_id = service.loadbalancer_id
        if loadbalancer_id in self.services:
            del(self.services[loadbalancer_id])

    def remove_by_loadbalancer_id(self, loadbalancer_id):
        """Remove service by providing the loadbalancer id."""
        if loadbalancer_id in self.services:
            del(self.services[loadbalancer_id])

    def get_by_loadbalancer_id(self, loadbalancer_id):
        """Retreive service by providing the loadbalancer id."""
        if loadbalancer_id in self.services:
            return self.services[loadbalancer_id]
        else:
            return None

    def get_loadbalancer_ids(self):
        """Return a list of cached loadbalancer ids."""
        return self.services.keys()

    def get_tenant_ids(self):
        """Return a list of tenant ids in the service cache."""
        tenant_ids = {}
        for service in self.services:
            tenant_ids[service.tenant_id] = 1
        return tenant_ids.keys()

    def get_agent_hosts(self):
        """Return a list of agent ids stored in the service cache."""
        agent_hosts = {}
        for service in self.services:
            agent_hosts[service.agent_host] = 1
        return agent_hosts.keys()

    def error_inc(self, lb_id):
        service = self.get_by_loadbalancer_id(lb_id)
        if service:
            service.error += 1

    def error_clear(self, lb_id):
        service = self.get_by_loadbalancer_id(lb_id)
        if service:
            service.error = 0

    def error_check(self, lb_id, try_err):
        service = self.get_by_loadbalancer_id(lb_id)
        if service:
            return service.error >= try_err
        else:
            return False

class LbaasAgentManager(periodic_task.PeriodicTasks):

    RPC_API_VERSION = '1.0'

    target = oslo_messaging.Target(version='1.0')

    def __init__(self, conf):
        if version == 'k':
            super(LbaasAgentManager, self).__init__()
        else:
            super(LbaasAgentManager, self).__init__(conf)
        LOG.debug("initialize LbaasAgentManager")

        self.conf = conf
        self.context = ncontext.get_admin_context_without_session()
        self.serializer = None

        self.cache = LogicalServiceCache()
        self.last_resync = datetime.datetime.now()
        self.needs_resync = False
        self.plugin_rpc = None
        self.start_time = True
        self.config_time = datetime.datetime.now()

        self.service_resync_interval = conf.service_resync_interval
        LOG.debug('setting service resync intervl to %d seconds' %
                  self.service_resync_interval)

        self.agent_host = None
        self._load_driver(conf)

        agent_configurations = (
            {'environment_prefix': self.conf.environment_prefix,
             'environment_group_number': self.conf.environment_group_number,
             'global_routed_mode': False}
        )
        self.agent_state = {
            'binary': constants_v2.AGENT_BINARY_NAME,
            'host': self.agent_host,
            'topic': constants_v2.TOPIC_LOADBALANCER_AGENT_V2,
            'configurations': agent_configurations,
            'agent_type': lb_const.AGENT_TYPE_LOADBALANCERV2,
            'l2_population': self.conf.l2_population,
            'start_flag': True
        }

        self.admin_state_up = True
        self.lbdriver.set_context(self.context)
        self._setup_rpc()
        self.lbdriver.post_init()
        self.needs_resync = True

        pass

    def config_time_update(self):
        self.config_time = datetime.datetime.now()

    def config_time_passed(self):
        return (datetime.datetime.now() - self.config_time).seconds

    def _load_driver(self, conf):
        self.lbdriver = None

        LOG.debug("loading LBaaS %s" % conf.sangfor_ad_lbaas_device_driver)

        try:
            self.lbdriver = importutils.import_object(
                    conf.sangfor_ad_lbaas_device_driver, self.conf)
            self.agent_host = '%s_%s' % (conf.host, self.conf.environment_prefix)
            self.lbdriver.agent_host = self.agent_host
            LOG.debug('setting agent host to %s' % self.agent_host)
        except ImportError as ie:
            msg = ('Error importing loadbalancer device driver: %s error %s'
                   % (conf.sangfor_ad_lbaas_device_driver, repr(ie)))
            LOG.error(msg)
            raise SystemExit(msg)
        pass

    def _setup_rpc(self):
        topic = constants_v2.TOPIC_PROCESS_ON_HOST_V2
        topic = topic + '_' + self.conf.environment_prefix
        LOG.debug('agent in %s environment will send callbacks to %s'
                % (self.conf.environment_prefix, topic))

        self.plugin_rpc = plugin_rpc.LBaaSv2PluginRPC(
            topic,
            self.context,
            self.conf.environment_prefix,
            self.conf.environment_group_number,
            self.agent_host
        )

        self.lbdriver.set_plugin_rpc(self.plugin_rpc)
        self._setup_state_rpc(topic)
        self.lbdriver.set_tunnel_rpc(agent_rpc.PluginApi(topics.PLUGIN))

        self._setup_loopingcall()

        consumers = [[constants_v2.TUNNEL, topics.UPDATE]]

        #L2 Populate
        self.lbdriver.set_l2pop_rpc(l2pop_rpc.L2populationAgentNotifyAPI())
        consumers.append([topics.L2POPULATION, topics.UPDATE, self.agent_host])

        self.endpoints = [self]
        self.connection = agent_rpc.create_consumers(self.endpoints, topics.AGENT, consumers)
        pass

    def _setup_state_rpc(self, topic):
        self.state_rpc = agent_rpc.PluginReportStateAPI(topic)
        report_interval = self.conf.AGENT.report_interval
        if report_interval:
            heartbeat = loopingcall.FixedIntervalLoopingCall(
                self._report_state)
            heartbeat.start(interval=report_interval)
        pass

    def _setup_loopingcall(self):
        if self.conf.config_check_enable:
            loopingcall.FixedIntervalLoopingCall(self.periodic_config_check).start(interval=self.conf.config_check_period)

    def _report_state(self):
        try:
            # assure the agent is connected
            # FIXME: what happens if we can't connect.
            #if not self.lbdriver.connected:
            #    self.lbdriver.connect()

            service_count = self.cache.size
            self.agent_state['configurations']['services'] = service_count
            if hasattr(self.lbdriver, 'service_queue'):
                self.agent_state['configurations']['request_queue_depth'] = (
                    len(self.lbdriver.service_queue)
                )

            # Add configuration from icontrol_driver.
            if self.lbdriver.agent_configurations:
                self.agent_state['configurations'].update(
                    self.lbdriver.agent_configurations
                )

            # Compute the capacity score.
            self.agent_state['configurations'][
                 'environment_capacity_score'] = 0

            LOG.debug("reporting state of agent as: %s" % self.agent_state)
            self.state_rpc.report_state(self.context, self.agent_state)
            self.agent_state.pop('start_flag', None)
        except Exception as e:
            LOG.exception(("Failed to report state: " + str(e.message)))
        pass

    def initialize_service_hook(self, started_by):
        node_topic = "%s_%s.%s" % (constants_v2.TOPIC_LOADBALANCER_AGENT_V2,
                                   self.conf.environment_prefix,
                                   self.agent_host)
        LOG.debug("Creating topic for consuming messages: %s" % node_topic)
        endpoints = [started_by.manager]
        started_by.conn.create_consumer(
            node_topic, endpoints, fanout=False)
        pass

    def check_pending_one(self, service, k):
        if k in service:
            if "provisioning_status" in service[k]:
                status = service[k]["provisioning_status"]
                if status in (plugin_const.PENDING_CREATE, plugin_const.PENDING_UPDATE, plugin_const.PENDING_DELETE):
                    LOG.debug("%s of service in %s status"%(k, status))
                    return True
        return False

    def check_pending_more(self, service, k):
        if k in service:
            for m in service[k].values():
                status = m["provisioning_status"]
                if status in (plugin_const.PENDING_CREATE, plugin_const.PENDING_UPDATE, plugin_const.PENDING_DELETE):
                    LOG.debug("%s of service in %s status" % (k, status))
                    return True
        return False

    def check_pending(self, service):
        if self.check_pending_one(service, "loadbalancer"):
            return True
        elif self.check_pending_more(service, "listeners"):
            return True
        elif self.check_pending_more(service, "pools"):
            return True
        elif self.check_pending_more(service, "healthmonitors"):
            return True
        elif self.check_pending_more(service, "members"):
            return True
        return False

    def periodic_config_check(self):
        LOG.debug("[PERIODIC_CONFIG_CHECK] start")
        try:
            loadbalancers = self.plugin_rpc.get_all_loadbalancers()
            idset = [e['lb_id'] for e in loadbalancers if e['agent_host'] == self.agent_host]

            LOG.debug("[periodic_task_service] valid loadbalancers %s for host %s" % (idset, self.agent_host))
            for i in idset:
                try:
                    if self.config_time_passed() < self.conf.service_config_interval:
                        LOG.debug("Config Update Now, Try Later")
                        return

                    self.cache.error_clear(i)

                    service = self.plugin_rpc.get_service_by_loadbalancer_id(i)
                    if not service:
                        continue

                    if self.check_pending(service) == True:
                        LOG.debug("In Pending State, No Needs To Check")
                        lb = service['loadbalancer']
                        self.cache.remove_by_loadbalancer_id(lb['id'])
                        continue

                    if self.lbdriver.sync_check_service(service) == False:
                        LOG.error("[periodic_task] Check Service Failed")

                except Exception as e:
                    LOG.error("[periodic_task] check for %s failed: %s" % (i, traceback.format_exc()))

            loadbalancers = self.plugin_rpc.get_all_loadbalancers()
            idset = [e['lb_id'] for e in loadbalancers]
            LOG.debug("[periodic_task_uuid] check valid loadbalancers %s on host %s" % (idset, self.agent_host))
            self.lbdriver.sync_check_uuid(idset)
        except Exception as e:
            LOG.debug("[periodic_task] catch exception: %s" % traceback.format_exc())

        LOG.debug("[PERIODIC_CONFIG_CHECK] end")

    @periodic_task.periodic_task(spacing=30)
    def periodic_agent_check(self, context):
        LOG.debug("[periodic_task] agent check start")
        self.plugin_rpc.scrub_dead_agents(self.conf.environment_prefix,
                                          self.conf.environment_group_number)
        pass

    @periodic_task.periodic_task(spacing=30)
    def periodic_update_tunnel(self, context):
        """
        定义获取vteps并更新AD，以后可以考虑agent注册注销的时候立即更新一次
        :return:
        """
        tunnel_types = self.agent_state['configurations']['tunnel_types']
        if 'vxlan' not in tunnel_types:
            return
        # 获取ad的vxlan配置并和plugin比较，如果不一致则更新
        # 另外，将AD的vteps通知给openstack其它模块
        self.lbdriver.tunnel_sync()

    @periodic_task.periodic_task
    def periodic_resync(self, context):
        LOG.debug("periodic_task called")
        now = datetime.datetime.now()
        if (now - self.last_resync).seconds > self.service_resync_interval:
            LOG.debug("no sync")
            if not self.needs_resync:
                self.needs_resync = True
                self.last_resync = now
        if self.needs_resync:
            self.needs_resync = False
            LOG.debug("sync")
            if self.sync_state():
                self.needs_resync = True
        pass

    @periodic_task.periodic_task(spacing=60)
    def periodic_member_status_report(self, context):
        def __get_pool_name(pool):
            if pool.get("name", "") != "":
                return "%s-%s" % (pool['name'], pool['id'])
            return "openstack-" + pool['id']

        try:
            active_lbs = (self.plugin_rpc.get_active_loadbalancers())
            lb_set = set()
            for lb in active_lbs:
                if self.agent_host == lb['agent_host']:
                    lb_set.add(lb['lb_id'])
            LOG.debug("Active Loadbalancer IDs %s" % lb_set)

            members = {}
            for i in lb_set:
                service = self.plugin_rpc.get_service_by_loadbalancer_id(i)
                if 'pools' not in service:
                    LOG.debug("Loadbalancer %s Has No Pool" % i)
                    continue
                for m in service['members'].values():
                    mip = self.lbdriver.netaddr_format(m['address'])
                    k = __get_pool_name(service['pools'][m['pool_id']])+':'+mip+':'+str(m['protocol_port'])
                    members[k] = m
            nodes = self.lbdriver.get_all_member_status()
            LOG.debug("get all member status %s" % nodes)
            for n in nodes:
                if n['key'] not in members:
                    LOG.error("Node %s Match Failed" % n["key"])
                    continue
                member = members[n['key']]
                if n['status'] != member['operating_status']:
                    LOG.debug("Update Status %s For Member %s" % (member['id'], n['status']))
                    self.plugin_rpc.update_member_status(member['id'], member['provisioning_status'], n['status'])
        except:
            LOG.error("Traceback: %s" % traceback.format_exc())

    def tunnel_sync(self):
        pass

    @log_helpers.log_method_call
    def sync_state(self):
        if not self.plugin_rpc:
            return
        resync = False
        known_services = set()
        for lb_id, service in self.cache.services.iteritems():
            if self.agent_host == service.agent_host:
                known_services.add(lb_id)
        try:
            active_loadbalancers = (
                self.plugin_rpc.get_active_loadbalancers()
            )
            active_loadbalancer_ids = set()
            for loadbalancer in active_loadbalancers:
                if self.agent_host == loadbalancer['agent_host']:
                    active_loadbalancer_ids.add(loadbalancer['lb_id'])
            LOG.debug("plugin produced the list of active loadbalancer ids: %s"
                      % list(active_loadbalancer_ids))
            LOG.debug("currently known loadbalancer ids before sync are: %s"
                      % list(known_services))
            for lb_id in active_loadbalancer_ids:
                if not self.cache.get_by_loadbalancer_id(lb_id):
                    self.validate_service(lb_id)
            pending_loadbalancers = (
                self.plugin_rpc.get_pending_loadbalancers()
            )
            pending_lb_ids = set()
            for loadbalancer in pending_loadbalancers:
                if self.agent_host == loadbalancer['agent_host']:
                    pending_lb_ids.add(loadbalancer['lb_id'])
            LOG.debug(
                "plugin produced the list of pending loadbalancer ids: %s"
                % list(pending_lb_ids))
            for lb_id in pending_lb_ids:
                self.refresh_service(lb_id)
            known_services = set()
            for (lb_id, service) in self.cache.services.iteritems():
                if self.agent_host == service.agent_host:
                    known_services.add(lb_id)
            LOG.debug("currently known loadbalancer ids after sync: %s"
                      % list(known_services))
        except Exception as e:
            LOG.error("Unable to retrieve ready service: %s" % e.message)
            resync = True
        return resync
        pass

    @log_helpers.log_method_call
    def validate_service(self, lb_id):
        if not self.plugin_rpc:
            return
        try:
            service = self.plugin_rpc.get_service_by_loadbalancer_id(
                lb_id
            )
            self.cache.put(service, self.agent_host)
            if self.check_pending(service) or not self.lbdriver.exists(service):
                LOG.error('pending loadbalancer %s is not on AD...syncing' % lb_id)
                self.lbdriver.sync(service)
        except q_exception.NeutronException as exc:
            LOG.error("NeutronException: %s" % traceback.format_exc())
        except Exception:
            LOG.error("Exception caught: validate_service, unable to validate",
                      " service for loadbalancer: %s" % traceback.format_exc())
        pass

    @log_helpers.log_method_call
    def refresh_service(self, lb_id):
        try:
            if self.cache.error_check(lb_id, self.conf.sync_try_times):
                LOG.debug("sync failed too much, stop sync %s" % lb_id)
                return

            service = self.plugin_rpc.get_service_by_loadbalancer_id(
                lb_id
            )
            if service.get('loadbalancer'):
                self.cache.put(service, self.agent_host)
                self.lbdriver.sync(service)
        except q_exception.NeutronException as exc:
            LOG.error("NeutronException: %s" % traceback.format_exc())
        except Exception as e:
            LOG.error("Exception: %s" % traceback.format_exc())
            self.needs_resync = True
        pass

    @log_helpers.log_method_call
    def destroy_service(self, lb_id):
        service = self.plugin_rpc.get_service_by_loadbalancer_id(
            lb_id
        )
        if not service:
            return
        service['loadbalancer']['provisioning_status'] = (
            plugin_const.PENDING_DELETE
        )
        try:
            self.lbdriver.delete_loadbalancer(lb_id, service)
        except q_exception.NeutronException as exc:
            LOG.error("NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
            self.needs_resync = True
        self.cache.remove_by_loadbalancer_id(lb_id)
        pass

    def remove_orphans(self, all_loadbalancers):
        pass

    def create_loadbalancer(self, context, loadbalancer, service):
        try:
            self.config_time_update()
            self.lbdriver.create_loadbalancer(loadbalancer, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("q_exception.NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def update_loadbalancer(self, context, old_loadbalancer,
                            loadbalancer, service):
        try:
            self.config_time_update()
            self.lbdriver.update_loadbalancer(old_loadbalancer,
                                              loadbalancer, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("q_exception.NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def delete_loadbalancer(self, context, loadbalancer, service):
        try:
            self.config_time_update()
            self.lbdriver.delete_loadbalancer(loadbalancer, service)
            self.cache.remove_by_loadbalancer_id(loadbalancer['id'])
        except q_exception.NeutronException as exc:
            LOG.error("q_exception.NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())

    def create_listener(self, context, listener, service):
        try:
            self.config_time_update()
            self.lbdriver.create_listener(listener, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("q_exception.NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def update_listener(self, context, old_listener, listener, service):
        try:
            self.config_time_update()
            self.lbdriver.update_listener(old_listener, listener, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("q_exception.NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def delete_listener(self, context, listener, service):
        try:
            self.config_time_update()
            self.lbdriver.delete_listener(listener, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("delete_listener: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("delete_listener: Exception: %s" % traceback.format_exc())

    def create_pool(self, context, pool, service):
        try:
            self.config_time_update()
            self.lbdriver.create_pool(pool, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def update_pool(self, context, old_pool, pool, service):
        try:
            self.config_time_update()
            self.lbdriver.update_pool(old_pool, pool, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def delete_pool(self, context, pool, service):
        """Handle RPC cast from plugin to update_pool."""
        try:
            self.config_time_update()
            self.lbdriver.delete_pool(pool, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("delete_pool: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("delete_pool: Exception: %s" % traceback.format_exc())

    def create_member(self, context, member, service):
        try:
            self.config_time_update()
            self.lbdriver.create_member(member, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("create_member: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("create_member: Exception: %s" % traceback.format_exc())
        pass

    def update_member(self, context, old_member, member, service):
        try:
            self.config_time_update()
            self.lbdriver.update_member(old_member, member, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("update_member: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("update_member: Exception: %s" % traceback.format_exc())
        pass

    def delete_member(self, context, member, service):
        try:
            self.config_time_update()
            self.lbdriver.delete_member(member, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("delete_member: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("delete_member: Exception: %s" % traceback.format_exc())

    def create_health_monitor(self, context, health_monitor, service):
        try:
            self.config_time_update()
            self.lbdriver.create_health_monitor(health_monitor, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("create_pool_health_monitor: NeutronException: %s"
                      % traceback.format_exc())
        except Exception as exc:
            LOG.error("create_pool_health_monitor: Exception: %s"
                      % traceback.format_exc())
        pass

    def update_health_monitor(self, context, old_health_monitor,
                              health_monitor, service):
        try:
            self.config_time_update()
            self.lbdriver.update_health_monitor(old_health_monitor,
                                                health_monitor,
                                                service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("update_health_monitor: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("update_health_monitor: Exception: %s" % traceback.format_exc())
        pass

    def delete_health_monitor(self, context, health_monitor, service):
        try:
            self.config_time_update()
            self.lbdriver.delete_health_monitor(health_monitor, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("delete_health_monitor: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("delete_health_monitor: Exception: %s" % traceback.format_exc())

    def create_l7policy(self, context, l7policy, service):
        try:
            self.config_time_update()
            self.lbdriver.create_l7policy(l7policy, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("q_exception.NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def update_l7policy(self, context, old_l7policy, l7policy, service):
        try:
            self.config_time_update()
            self.lbdriver.update_l7policy(old_l7policy, l7policy, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("q_exception.NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def delete_l7policy(self, context, l7policy, service):
        try:
            self.config_time_update()
            self.lbdriver.delete_l7policy(l7policy, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("delete_l7policy: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("delete_l7policy: Exception: %s" % traceback.format_exc())
        pass

    def create_l7rule(self, context, l7rule, service):
        try:
            self.config_time_update()
            self.lbdriver.create_l7rule(l7rule, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("q_exception.NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def update_l7rule(self, context, old_l7rule, l7rule, service):
        try:
            self.config_time_update()
            self.lbdriver.update_l7rule(old_l7rule, l7rule, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("q_exception.NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("Exception: %s" % traceback.format_exc())
        pass

    def delete_l7rule(self, context, l7rule, service):
        try:
            self.config_time_update()
            self.lbdriver.delete_l7rule(l7rule, service)
            self.cache.put(service, self.agent_host)
        except q_exception.NeutronException as exc:
            LOG.error("delete_l7rule: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("delete_l7rule: Exception: %s" % traceback.format_exc())
        pass

    @log_helpers.log_method_call
    def agent_updated(self, context, payload):
        """Handle the agent_updated notification event."""
        if payload['admin_state_up'] != self.admin_state_up:
            self.admin_state_up = payload['admin_state_up']
            if self.admin_state_up:
                # FIXME: This needs to be changed back to True
                self.needs_resync = False
            else:
                for loadbalancer_id in self.cache.get_loadbalancer_ids():
                    LOG.debug("DESTROYING loadbalancer: " + loadbalancer_id)
                    # self.destroy_service(loadbalancer_id)
            LOG.info("agent_updated by server side %s!", payload)
        pass

    @log_helpers.log_method_call
    def tunnel_update(self, context, **kwargs):
        """Handle RPC cast from core to update tunnel definitions."""
        try:
            LOG.debug('received tunnel_update: %s' % kwargs)
            self.lbdriver.tunnel_update(**kwargs)
        except q_exception.NeutronException as exc:
            LOG.error("tunnel_update: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("tunnel_update: Exception: %s" % traceback.format_exc())
        pass

    @log_helpers.log_method_call
    def add_fdb_entries(self, context, fdb_entries, host=None):
        """Handle RPC cast from core to update tunnel definitions."""
        try:
            LOG.debug('received add_fdb_entries: %s host: %s'
                      % (fdb_entries, host))
            #self.lbdriver.fdb_add(fdb_entries)
        except q_exception.NeutronException as exc:
            LOG.error("fdb_add: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("fdb_add: Exception: %s" % traceback.format_exc())
        pass

    @log_helpers.log_method_call
    def remove_fdb_entries(self, context, fdb_entries, host=None):
        try:
            LOG.debug('received remove_fdb_entries: %s host: %s'
                      % (fdb_entries, host))
            #self.lbdriver.fdb_remove(fdb_entries)
        except q_exception.NeutronException as exc:
            LOG.error("remove_fdb_entries: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("remove_fdb_entries: Exception: %s" % traceback.format_exc())
        pass
    @log_helpers.log_method_call
    def update_fdb_entries(self, context, fdb_entries, host=None):
        try:
            LOG.debug('received update_fdb_entries: %s host: %s'
                      % (fdb_entries, host))
            #self.lbdriver.fdb_update(fdb_entries)
        except q_exception.NeutronException as exc:
            LOG.error("update_fdb_entrie: NeutronException: %s" % traceback.format_exc())
        except Exception as exc:
            LOG.error("update_fdb_entrie: Exception: %s" % traceback.format_exc())
        pass
